package net.shrine.adapter.translators

import net.shrine.adapter.mappings.AdapterMappings
import net.shrine.log.Loggable
import net.shrine.protocol.query.Expression
import scala.util.Try

/**
 * @author Clint Gilbert
 * @since Feb 29, 2012
 *
 * A class to translate query Expressions from Shrine terms to local terms
 * given an AdapterMappings instance.
 */
final case class ExpressionTranslator(translate: Seq[String] => Map[String, Set[String]]) extends Loggable {

  def translate(expr: Expression): Expression = tryTranslate(expr).get.normalize
  
  //NB: Package-private for testing
  private[translators] def tryTranslate(expr: Expression): Try[Expression] = Expression.translate(expr, translate)
}

object ExpressionTranslator {
  def apply(mappings: Map[String, Set[String]]): ExpressionTranslator =
    ExpressionTranslator((vs: Seq[String]) => vs.map(v => v -> mappings.getOrElse(v, Set())).toMap)

  def apply(adapterMappings: AdapterMappings): ExpressionTranslator =
    ExpressionTranslator((vs: Seq[String]) => adapterMappings.localTermsFor(vs))
}