package net.shrine.http4s.servlet.syntax

import cats.effect.{ConcurrentEffect, ContextShift}
import javax.servlet.{ServletContext, ServletRegistration}
import net.shrine.source.ConfigSource
import org.http4s.HttpRoutes
import org.http4s.server.DefaultServiceErrorHandler
import org.http4s.servlet.{AsyncHttp4sServlet, NonBlockingServletIo}
import net.shrine.config.ConfigExtensions

import scala.concurrent.duration.Duration
import scala.language.higherKinds

object ShrineServletContextSyntax {

  /** Wraps an [[HttpRoutes]] and mounts it as an [[AsyncHttp4sServlet]]
    *
    * Assumes non-blocking servlet IO is available, and thus requires at least Servlet 3.1.
    *
    * This is copied more or less directly from org.http4s.servlet.syntax.ServletContextSyntax
    * in http4s 0.19 M2, and may need updating with future releases.
    */
  def mountService[F[_]: ConcurrentEffect: ContextShift](
                                                          context: ServletContext,
                                                          name: String,
                                                          service: HttpRoutes[F],
                                                          mapping: String = "/*"): ServletRegistration.Dynamic = {
    val servlet = new AsyncHttp4sServlet(
      service = service,
      asyncTimeout = ConfigSource.config.get("shrine.api.asyncTimeout",Duration(_)),
      servletIo = NonBlockingServletIo(4096),
      serviceErrorHandler = DefaultServiceErrorHandler[F]
    )
    val reg = context.addServlet(name, servlet)
    reg.setLoadOnStartup(1)
    reg.setAsyncSupported(true)
    reg.addMapping(mapping)
    reg
  }
}

