package net.shrine.testapp

import cats.effect.{ConcurrentEffect, ContextShift, IO}
import javax.servlet.annotation.WebListener
import javax.servlet.{ServletContext, ServletContextEvent, ServletContextListener, ServletRegistration}
import org.http4s.server.DefaultServiceErrorHandler
import org.http4s.servlet.{AsyncHttp4sServlet, BlockingServletIo, NonBlockingServletIo}

import scala.concurrent.duration.Duration
import org.http4s.HttpRoutes
import org.http4s.dsl.impl.{->, /}
import org.http4s.dsl.io.{GET, Ok, PUT, Root, http4sOkSyntax}

import scala.language.higherKinds
import scala.concurrent.ExecutionContext.Implicits.global

/**
  * A simple servlet to bracket problems in Shrine without bringing in all of Shrine's complexity
  */

@WebListener
class TestAppBootstrap extends ServletContextListener {

  override def contextInitialized(sce: ServletContextEvent): Unit = {
    val context = sce.getServletContext

    implicit val contextShift: ContextShift[IO] = IO.contextShift(global)
    implicit val concurrentEffect: ConcurrentEffect[IO] = IO.ioConcurrentEffect(contextShift)

    mountService(
      context = context,
      name = "test http4s",
      service = service,
      mapping = "/*"
    )
  }

  def mountService[F[_]: ConcurrentEffect: ContextShift](
                    context: ServletContext,
                    name: String,
                    service: HttpRoutes[F],
                    mapping: String): ServletRegistration.Dynamic = {

    val servlet = new AsyncHttp4sServlet(
      service = service,
      asyncTimeout = Duration("30 seconds"),
//      servletIo = NonBlockingServletIo(4096), //limited to 4096 bytes per request
      servletIo = BlockingServletIo(4096,global), //works
      serviceErrorHandler = DefaultServiceErrorHandler[F]
    )
    val reg = context.addServlet(name, servlet)
    reg.setLoadOnStartup(1)
    reg.setAsyncSupported(true)
    reg.addMapping(mapping)
    reg
  }

  val service: HttpRoutes[IO] = HttpRoutes.of[IO] {

    case _@GET -> Root / "ping" => Ok("pong")

    case sizeReq@PUT -> Root / "size" =>
      val messageTextIO: IO[String] = sizeReq.as[String]
      messageTextIO.flatMap{text => Ok(s"Received ${text.length}")}
  }

  override def contextDestroyed(sce: ServletContextEvent): Unit = {}
}

