package net.shrine.authentication

import net.shrine.http4s.client.legacy.Poster
import net.shrine.i2b2.protocol.pm.{GetUserConfigurationRequest, User}
import net.shrine.protocol.AuthenticationInfo

import scala.util.Try
import scala.util.control.NonFatal

/**
 * @author clint
 * @since Feb 26, 2014
 */
abstract class BasePmAuthenticator(pmPoster: Poster, makeAuthenticationResult: (AuthenticationInfo, User) => AuthenticationResult) extends Authenticator {

    final override def authenticate(authn: AuthenticationInfo, onAuth: Option[User => Unit] = None): AuthenticationResult = {
    val httpResponseString = Try {
      val requestString = GetUserConfigurationRequest(authn).toI2b2String

      val httpResponse = pmPoster.post(requestString)

      httpResponse.body
    }

    val userFromPm = httpResponseString.flatMap(User.fromI2b2)

    val result = userFromPm.map { user =>
      onAuth.foreach(_(user))
      makeAuthenticationResult(authn, user)
    }.recover {
      case NonFatal(e) => {
        val AuthenticationInfo(domain, username, _) = authn

        AuthenticationResult.NotAuthenticated(domain, username, s"Failed attempt to certify user", Some(e))
      }
    }
    
    result.get
  }
}