package net.shrine.authorization

import net.shrine.log.Loggable
import net.shrine.protocol.ReadApprovedQueryTopicsRequest
import net.shrine.protocol.ReadApprovedQueryTopicsResponse
import net.shrine.protocol.RunQueryRequest
import AuthorizationResult.Authorized
import net.shrine.i2b2.protocol.pm.User
import net.shrine.protocol.ErrorResponse
import net.shrine.protocol.query.QueryDefinition

/**
 * @author Bill Simons
 * @since Aug 25, 2010
 * @see http://cbmi.med.harvard.edu
 * @see http://chip.org
 * <p/>
 * NOTICE: This software comes with NO guarantees whatsoever and is
 * licensed as Lgpl Open Source
 * @see http://www.gnu.org/licenses/lgpl.html
 */
object AllowsAllAuthorizationService extends QueryAuthorizationService with Loggable {
  override def authorizeRunQueryRequest(request: RunQueryRequest): AuthorizationResult = {
    if(request == null) {
      //todo clean out nulls
      val npe = new NullPointerException("request is null in AllowsAllAuthorizationService.authorizeRunQueryRequest()")
      npe.fillInStackTrace()
      warn(s"request is null in authorizeRunQueryRequest",npe)
      Authorized(None)
    }
    else {
      val topicIdAndName = (request.topicId, request.topicName) match {
        case (Some(id), Some(name)) => Option((id, name))
        case (None, None) => None
        case _ => throw new IllegalArgumentException(s"(request.topicId, request.topicName) are ${(request.topicId, request.topicName)}")
      }
      Authorized(topicIdAndName)
    }
  }

  override def authorizeRunQuery(user: User, networkQueryId: Long, queryDefinition: QueryDefinition, topicId: Option[String], topicName: Option[String]): AuthorizationResult = {

    val topicIdAndName = (topicId, topicName) match {
      case (Some(id), Some(name)) => Option((id.toString, name))
      case (None, None) => None
      case topic => throw new IllegalArgumentException(s"(request.topicId, request.topicName) are ${(topic._1, topic._2)}")
    }
    Authorized(topicIdAndName)
  }

  override def readApprovedTopics(request: ReadApprovedQueryTopicsRequest): Either[ErrorResponse, ReadApprovedQueryTopicsResponse] = {
    //TODO: This is a smell
    throw new UnsupportedOperationException
  }
}
