package net.shrine.authorization

import net.shrine.i2b2.protocol.pm.User
import net.shrine.protocol.query.QueryDefinition
import net.shrine.protocol.{ErrorResponse, ReadApprovedQueryTopicsRequest, ReadApprovedQueryTopicsResponse, RunQueryRequest}

/**
 * @author Bill Simons
 * @since Aug 24, 2010
 * @see http://cbmi.med.harvard.edu
 * @see http://chip.org
 * <p/>
 * NOTICE: This software comes with NO guarantees whatsoever and is
 * licensed as Lgpl Open Source
 * @see http://www.gnu.org/licenses/lgpl.html
 */
trait QueryAuthorizationService {

  //Contact a data steward and either return an Authorized or a NotAuthorized or throw an exception
  @throws(classOf[AuthorizationException])
  def authorizeRunQueryRequest(request: RunQueryRequest): AuthorizationResult

  def authorizeRunQuery(user: User, networkQueryId: Long, queryDefinition: QueryDefinition, topicId: Option[String], topicName: Option[String]): AuthorizationResult

  //Either read the approved topics from a data steward or have an error response.
  def readApprovedTopics(request: ReadApprovedQueryTopicsRequest): Either[ErrorResponse, ReadApprovedQueryTopicsResponse]
}
