package net.shrine.crypto

import java.security.{Principal, PrivateKey, PublicKey}
import java.security.cert.X509Certificate


/**
  * Allows gradual replacement of the old crypto package by keeping the old
  * interface for now
  */
//TODO: Delete when we no longer need CertCollection - SHRINE-3425
final case class CertCollectionAdapter(keyStoreCollection: BouncyKeyStoreCollection)
  extends BouncyKeyStoreCollection 
{
  override def signBytes(bytesToSign: Array[Byte]): Array[Byte] = keyStoreCollection.signBytes(bytesToSign)

  override def verifyBytes(cmsEncodedSignature: Array[Byte], originalMessage: Array[Byte]): Boolean = keyStoreCollection.verifyBytes(cmsEncodedSignature, originalMessage)

  override def verifySignature(cmsEncodedSignature: Array[Byte], originalMessage: Array[Byte]): Either[Throwable, Unit] = keyStoreCollection.verifySignature(cmsEncodedSignature, originalMessage)

  override val myEntry: KeyStoreEntry = keyStoreCollection.myEntry

  override def allEntries: Iterable[KeyStoreEntry] = keyStoreCollection.allEntries

  def myCertId: Option[CertId] = Some(entryToCertId(myEntry))

  def myCert: Option[X509Certificate] = Some(myEntry.cert)

  def caCertAliases: Seq[String] = caEntry.aliases

  def caCerts: Map[Principal, X509Certificate] = Map(caEntry.cert.getIssuerDN -> caEntry.cert)

  def myKeyPair: KeyPair = KeyPair(myEntry.publicKey, myEntry.privateKey.get)

  def get(id: CertId): Option[X509Certificate] = certIdsToCerts.get(id)

  def iterator: Iterator[X509Certificate] = keyStoreCollection.allEntries.map(_.cert).iterator

  def ids: Iterable[CertId] = allEntries.filterNot(_ == caEntry).map(entryToCertId)

  def caIds: Iterable[CertId] = Seq(entryToCertId(caEntry))

  // CertIds are just the serial number along with the alias
  private def entryToCertId(keyStoreEntry: KeyStoreEntry): CertId =
    CertId(keyStoreEntry.certificateHolder.getSerialNumber, Some(keyStoreEntry.aliases.first))

  private val certIdsToCerts: Map[CertId, X509Certificate] = keyStoreCollection.allEntries.map(entry => entryToCertId(entry) -> entry.cert).toMap

  // The CertCollection doesn't really account for PeerToPeer networks, so we slightly ignore that too
  private val caEntry: KeyStoreEntry = keyStoreCollection match {
    case DownStreamCertCollection(_, ca)     => ca
    case HubCertCollection(ca, _)            => ca
    case PeerCertCollection(privateEntry, _ ) => privateEntry
  }

}

case class KeyPair(publicKey: PublicKey, privateKey: PrivateKey)
