package net.shrine.crypto

import java.math.BigInteger

import net.shrine.serialization.{XmlMarshaller, XmlUnmarshaller}
import net.shrine.util.{OptionEnrichments, XmlUtil}

import scala.util.Try
import scala.xml.NodeSeq

/**
 * @author clint
 * @since Nov 22, 2013
 */
final case class CertId(serial: BigInteger, name: Option[String] = None) extends XmlMarshaller {
  override def toXml: NodeSeq = XmlUtil.stripWhitespace {
    import OptionEnrichments._
    Nil
    
    <certId>
      <serial>{ serial }</serial>
      { name.toXml(<name/>) }
    </certId>
  }

  override def equals(other: Any): Boolean = other match {
    case that: CertId => that.serial == this.serial
    case _ => false
  }

  override def hashCode: Int = serial.hashCode
}

object CertId extends XmlUnmarshaller[Try[CertId]] {
  override def fromXml(xml: NodeSeq): Try[CertId] = {
    for {
      serial <- Try(new BigInteger((xml \ "serial").text.trim))
      nameOption = (xml \ "name").headOption.map(_.text.trim)
    } yield CertId(serial, nameOption)
  }
}