package net.shrine.email

import java.util.Properties
import scala.collection.JavaConverters.{asScalaSet, mapAsJavaMap}

import com.typesafe.config.Config
import courier.Mailer
import javax.mail.{Authenticator, PasswordAuthentication, Session}
import net.shrine.config.ConfigExtensions

/**
  * Creates a courier Mailer via shrine.conf, by pulling out all possible properties from https://www.tutorialspoint.com/javamail_api/javamail_api_smtp_servers.htm from an email section of shrine.conf
  *
  * @author david 
  * @since 1.22
  */
object ConfiguredMailer {

  //for testing
  private[email] def createSessionFromConfig(config:Config):Session = {
    //convert the config to a java.util.Properties

    val properties = new Properties()

    val map: Map[String, Object] = asScalaSet(config.getConfig("javaxmail").entrySet()).map({ entry =>
      entry.getKey -> entry.getValue.unwrapped()
    })(collection.breakOut)
    
    properties.putAll(mapAsJavaMap(map))

    def authenticatorFromConfig(config: Config): Authenticator = {
      new javax.mail.Authenticator() {
        override def getPasswordAuthentication = new PasswordAuthentication(config.getString("username"), config.getString("password"))
      }
    }

    val configAuthenticator: Option[Authenticator] = config.getOptionConfigured("authenticator",authenticatorFromConfig)

    configAuthenticator.fold(Session.getDefaultInstance(properties))(
                    authenticator => Session.getDefaultInstance(properties,authenticator))
  }

  def createMailerFromConfig(config:Config):Mailer = {
    val session = createSessionFromConfig(config)

    Mailer(session)
  }
}
