package net.shrine.http4s.auth

import cats.data.{Kleisli, OptionT}
import cats.effect.IO
import net.shrine.i2b2.protocol.pm.User
import net.shrine.source.ConfigSource
import org.http4s.Credentials.Token
import org.http4s.headers.Authorization
import org.http4s.{AuthScheme, AuthedRequest, AuthedRoutes, BasicCredentials, HttpRoutes, Request, Response, Status}


object UserAuthentication {
  lazy val domain: String = ConfigSource.config.getString("shrine.webclient.domain")

  val researcherRole: String = "Researcher"
  val stewardRole: String = "DataSteward"
  val qepRole: String = "qep"

  def addUnauthorizedMsg(message: String) =
    Kleisli { resp: Response[IO] =>
      OptionT[IO, Response[IO]] {
        if (resp.status == Status.Unauthorized)
          IO(Some(resp.withEntity(message)))
        else IO(Some(resp))
      }
    }
}

//Allows Either Basic or Bearer Authentication
object BasicOrBearerAuthMiddleware {

  def apply(service: AuthedRoutes[User, IO]): HttpRoutes[IO] = Kleisli({ request =>

    val authHeader = request.headers.get(Authorization)
    authHeader match {
      case Some(Authorization(Token(AuthScheme.Bearer, token))) => BearerAuthMiddleware(service)(request)
      case _ => BasicAuthMiddleware(service)(request)
    }
  })
}

