package net.shrine.http4s.servlet

import cats.effect.{ConcurrentEffect, ContextShift, IO}
import javax.servlet.{ServletContext, ServletContextEvent, ServletRegistration}
import net.shrine.source.ConfigSource
import org.http4s.HttpRoutes
import org.http4s.server.DefaultServiceErrorHandler
import org.http4s.servlet.{AsyncHttp4sServlet, BlockingServletIo, NonBlockingServletIo}
import net.shrine.config.ConfigExtensions
import net.shrine.http4s.catsio.ExecutionContexts

import scala.concurrent.duration.Duration
import scala.language.higherKinds

object ShrineServletContextSyntax {

  def mount(sce: ServletContextEvent,
            name: String,
            service: HttpRoutes[IO],
            mapping: String = "/*"): Unit = {
    val context = sce.getServletContext

    //TODO: find out why AutoSlash middleware does not work - SHRINE-2581

    val serviceWithLogging = Http4sRequestResponseLogger(service)

    implicit val contextShift: ContextShift[IO] = IO.contextShift(ExecutionContexts.http4sExecutionContext)
    implicit val concurrentEffect: ConcurrentEffect[IO] = IO.ioConcurrentEffect(contextShift)

    mountService(context, name, serviceWithLogging, mapping)
  }

  /** Wraps an [[HttpRoutes]] and mounts it as an [[AsyncHttp4sServlet]]
    *
    * Assumes non-blocking servlet IO is available, and thus requires at least Servlet 3.1.
    *
    * This is copied more or less directly from org.http4s.servlet.syntax.ServletContextSyntax
    * in http4s 0.19 M2, and may need updating with future releases.
    */
  def mountService[F[_]: ConcurrentEffect: ContextShift](
                                                          context: ServletContext,
                                                          name: String,
                                                          service: HttpRoutes[F],
                                                          mapping: String = "/*"): ServletRegistration.Dynamic = {
    val servlet = new AsyncHttp4sServlet(
      service = service,
      asyncTimeout = ConfigSource.config.get("shrine.api.asyncTimeout",Duration(_)),
//      servletIo = NonBlockingServletIo(4096), todo switch back to NonBlocking when https://github.com/http4s/http4s/issues/2354 is fixed
      servletIo = BlockingServletIo(4096,ExecutionContexts.http4sExecutionContext),
      serviceErrorHandler = DefaultServiceErrorHandler[F]
    )
    val reg = context.addServlet(name, servlet)
    reg.setLoadOnStartup(1)
    reg.setAsyncSupported(true)
    reg.addMapping(mapping)
    reg
  }
}