package net.shrine.protocol

import net.shrine.serialization.XmlMarshaller
import net.shrine.util.XmlUtil
import net.shrine.serialization.XmlUnmarshaller
import net.shrine.protocol.version.v1.{Node => ProtocolNode}

import scala.util.Try
import scala.xml.{Node, NodeSeq}

/**
 * @author clint
 * @since Nov 1, 2013
 */
final case class XmlNodeName(name: String) extends XmlMarshaller {
  override def toXml: Node = XmlUtil.stripWhitespace {
    import XmlNodeName._

    XmlUtil.renameRootTag(rootTagName) {
      <placeholder>
        <name>{ name }</name>
      </placeholder>
    }
  }
}

object XmlNodeName extends XmlUnmarshaller[Try[XmlNodeName]] {

  val rootTagName = "nodeId"

  val Unknown = XmlNodeName("Unknown")

  override def fromXml(xml: NodeSeq): Try[XmlNodeName] = {
    for {
      name <- Try((xml \ "name").text.trim).filter(!_.isEmpty)
    } yield XmlNodeName(name)
  }

  def fromXmlOption(xml:NodeSeq):Try[Option[XmlNodeName]] = Try {
    val nameElement: NodeSeq = (xml \ "name")
    val contents = nameElement.text
    if (contents.isEmpty) None
    else Some(XmlNodeName(contents))
  }

  def fromProtocolNode(protocolNode: ProtocolNode):XmlNodeName = {
    XmlNodeName(protocolNode.name.underlying)
  }
}
 