package net.shrine.protocol.version

import scala.util.Random


/**
  * Typechecked ids for various kinds of data in shrine
  *
  * @author david
  * @since 1.26
  */
trait Id extends Any with ValueClass[Long]

object Id {
  private[version] def create[I <: Id](longToI: Long => I):I = longToI(nextId())

  def nextId(): Long = synchronized(Math.abs(Random.nextLong()))
}

class ResultId(val underlying:Long) extends AnyVal with Id

object ResultId {
  private[version] def create():ResultId = Id.create(new ResultId(_))
}

class QueryId(val underlying:Long) extends AnyVal with Id

object QueryId {
  private[version] def create():QueryId = Id.create(new QueryId(_))
}

class TopicId(val underlying:Long) extends AnyVal with Id

object TopicId {
  private[version] def create():TopicId = Id.create(new TopicId(_))
}

class ResearcherId(val underlying:Long) extends AnyVal with Id

object ResearcherId {
  private[version] def create():ResearcherId = Id.create(new ResearcherId(_))
}
class NodeId(val underlying:Long) extends AnyVal with Id

object NodeId {
  private[version] def create():NodeId = Id.create(new NodeId(_))
}

class NetworkId(val underlying:Long) extends AnyVal with Id

object NetworkId {
  /**
    * The NetworkId is special because there's exactly one of them in a shrine network.
    */
  val oneNetwork:NetworkId = new NetworkId(1L)
}

