package net.shrine.protocol.version


import net.shrine.source.ConfigSource
import net.shrine.config.ConfigExtensions

/**
  * Marker trait for ValueTypes
  */
trait ValueClass[U] extends Any {
  def underlying:U

  override def toString: String = s"${this.getClass.getSimpleName}($underlying)"
}

//VersionInfo parts

class ProtocolVersion(val underlying:Int) extends AnyVal with ValueClass[Int] 

object ProtocolVersion {
  val current:ProtocolVersion = v1.versionId
}

class ItemVersion(val underlying:Int) extends AnyVal with ValueClass[Int]

object ItemVersion {
  val one:ItemVersion = new ItemVersion(1)
  def next(itemVersion: ItemVersion) = new ItemVersion(itemVersion.underlying+1)
}

class DateStamp(val underlying:Long) extends AnyVal with ValueClass[Long]

object DateStamp {
  def now = new DateStamp(System.currentTimeMillis())

  val repeatableFakeDateStamp:DateStamp = new DateStamp(0L)
}

class JsonText(val underlying:String) extends AnyVal with ValueClass[String]

//Other shared parts of different Shrine data
class NodeName(val underlying:String) extends AnyVal with ValueClass[String]

class NodeKey(val underlying:String) extends AnyVal with ValueClass[String] 

object NodeKey {
  val localNodeKey:NodeKey = ConfigSource.atomicConfig.config.get("shrine.nodeKey",new NodeKey(_))
}

class MomQueueName(val underlying:String) extends AnyVal with ValueClass[String] {
  //todo check that the underlying String name is a valid queue name - can be part of a URL path
}

class UserDomainName(val underlying:String) extends AnyVal with ValueClass[String]

class UserName(val underlying:String) extends AnyVal with ValueClass[String]