package net.shrine.protocol.version.v1

import io.circe.{Decoder, Encoder}
import net.shrine.protocol.version.{JsonText, NodeId, ResearcherId, UserDomainName, UserName, ValueClass}

import scala.util.Try


/**
  * A researcher in the shrine network
  *
  * @since 1.26
  * @author dwalend
  */
case class Researcher(
                 id:ResearcherId = ResearcherId.create(),
                 versionInfo: VersionInfo = VersionInfo.create,
                 userName:UserName,
                 userDomainName:UserDomainName,
                 nodeId:NodeId
                ) extends Versioned[ResearcherId] {

  def asJsonText:JsonText = {
    //todo this can maybe be fully generic for all the common currency data, in Versioned
    //todo but the generics didn't work out of the box. Ask on the circe gitter channel SHRINE-2848
    import io.circe.generic.extras.Configuration
    import io.circe.generic.extras.auto.exportEncoder
    import io.circe.syntax.EncoderOps

    implicit val genDevConfig: Configuration = Versioned.genDevConfig
    import io.circe.generic.extras.encoding.UnwrappedEncoder
    implicit def valueClassEncoder[A <: ValueClass[_]](implicit encodeA: UnwrappedEncoder[A]): Encoder[A] = encodeA

    new JsonText(this.asJson.noSpaces)
  }
}

object Researcher extends VersionedCompanion {

  val envelopeType: String = classOf[Researcher].getSimpleName

  def create(
            userName: String,
            userDomainName: String,
            nodeId: NodeId
            ) = Researcher(
    id = ResearcherId.create(),
    userName = new UserName(userName),
    userDomainName = new UserDomainName(userDomainName),
    nodeId = nodeId
  )

  def createRepeatableForV26(
              userName: String,
              userDomainName: String,
              nodeId: NodeId
            ) = Researcher(
    id = new ResearcherId(userName.hashCode  ^ userDomainName.hashCode ^ nodeId.underlying.hashCode()),
    versionInfo = VersionInfo.createWithRepeatableFakeDateStamp,
    userName = new UserName(userName),
    userDomainName = new UserDomainName(userDomainName),
    nodeId = nodeId
  )

  //todo if this can maybe be fully generic for all the common currency data, maybe it can live in Versioned SHRINE-2848
  def tryRead(jsonText:JsonText):Try[Researcher] = Try {
    import io.circe.parser.decode
    import io.circe.generic.extras.auto.exportDecoder
    import io.circe.generic.extras.Configuration

    implicit val genDevConfig: Configuration = Versioned.genDevConfig
    import io.circe.generic.extras.decoding.UnwrappedDecoder
    implicit def valueClassDecoder[A <: ValueClass[_]](implicit decodeA: UnwrappedDecoder[A]): Decoder[A] = decodeA

    decode[Researcher](jsonText.underlying) match {
      case Right(researcher) => researcher
      case Left(x) => throw x //throw errors for now. Eventually do something clever, maybe with circe optics
    }
  }
}
