package net.shrine.protocol.version.v1

import net.shrine.crypto.NewTestKeyStore
import net.shrine.protocol.ResultOutputType
import net.shrine.protocol.version.{JsonText, NodeId, NodeName, QueryId, ResearcherId, ResultId, ResultStatuses, TopicId, UserDomainName, UserName}
import org.junit.Test
import org.scalatestplus.junit.AssertionsForJUnit

import scala.collection.immutable.Seq

/**
  * Round-trip tests of common currency to and from json
  */
class JsonTest extends AssertionsForJUnit {

  val crcResult = CrcResult(
                            id = ResultId.create(),
                            versionInfo = VersionInfo.create,
                            queryId = QueryId.create(),
                            adapterNodeId = NodeId.create(),
                            adapterNodeName = new NodeName("test node"),
                            status = ResultStatuses.ResultFromCRC,
                            statusMessage = None,
                            crcQueryInstanceId = Some(3L),
                            count = 2050,
                            resultType = ResultOutputType.PATIENT_COUNT_XML,
                            breakdowns = Some(
                              Breakdowns(
                                Seq("colors" -> Seq(
                                                    "red" -> 300,
                                                    "blue" -> 600,
                                                    "green" -> 30
                            ))))
                          )

  @Test
  def testCrcResultJsonRoundTrip():Unit = {
    assertResult(crcResult){Result.tryRead(crcResult.asJsonText).get}
  }

  val resultProgress = ResultProgress(
    id = ResultId.create(),
    versionInfo = VersionInfo.create,
    queryId = QueryId.create(),
    adapterNodeId = NodeId.create(),
    adapterNodeName = new NodeName("test node"),
    status = ResultStatuses.IdAssigned,
    statusMessage = None
  )

  @Test
  def testResultProgressJsonRoundTrip():Unit = {
    assertResult(resultProgress){Result.tryRead(resultProgress.asJsonText).get}
  }

  @Test
  def testResearcherRoundTrip():Unit = {
    val researcher = Researcher(
      userName = new UserName("Isha"),
      userDomainName = new UserDomainName("hmscatalyst"),
      nodeId = NodeId.create()
    )

    assertResult(researcher){Researcher.tryRead(researcher.asJsonText).get}
  }

  @Test
  def testQueryRoundTrip():Unit = {
    val query = Query.create(
      queryDefinitionXml = "query xml",
      outputTypesXml = "query output types",
      queryName = "query Name",
      nodeOfOriginId = new NodeId(0),
      researcherId = new ResearcherId(1),
      topicId = new TopicId(2),
      projectName = "project name",
      certCollection = NewTestKeyStore.certCollection
    )

    val jsonString: JsonText = query.asJsonText

    val queryBack = Query.tryRead(jsonString).get
    assertResult(query){queryBack}

//todo why did this break? SHRINE-3613 assert(queryBack.verifySignature(NewTestKeyStore.certCollection).isRight)
  }

  @Test
  def testRunQueryForResultRoundTrip():Unit = {
    val node = Node.create("test node","testnode","testnode.edu", adminEmail="email@foo")

    val researcher = Researcher(
      userName = new UserName("Isha"),
      userDomainName = new UserDomainName("hmscatalyst"),
      nodeId = node.id
    )

    val topic = Topic.createRepeatableForV26(researcher.id,researcher.userName.underlying,researcher.userDomainName.underlying)

    val query = Query.create(
      queryDefinitionXml = "query xml",
      outputTypesXml = "query output types",
      queryName = "query Name",
      nodeOfOriginId = node.id,
      researcherId = researcher.id,
      topicId = topic.id,
      projectName = "project name",
      certCollection = NewTestKeyStore.certCollection
    )

    val result = Result.create(query,node)

    val runQueryForResult = RunQueryForResult(query,researcher,node,topic,result)

    assertResult(runQueryForResult){RunQueryForResult.tryRead(runQueryForResult.asJsonText).get}
  }

}