package net.shrine.protocol.version.v1

import net.shrine.problem.{JsonProblemDigest, TestProblem}
import net.shrine.protocol.ResultOutputType
import net.shrine.protocol.version.{JsonText, ResultStatuses}
import net.shrine.util.ShouldMatchersForJUnit
import org.junit.Test

class ResultTest extends ShouldMatchersForJUnit {

  val jsonResultProgress = V1JsonTest.readJsonFile("/resultProgress.json")
  val jsonCrcResult = V1JsonTest.readJsonFile("/crcResult.json")
  val jsonErrorResult = V1JsonTest.readJsonFile("/errorResult.json")

  val expectedResultProgress = V1JsonTest.staticResultProgress

  val expectedCrcResult = CrcResult(
    id = V1JsonTest.staticResultId,
    versionInfo = V1JsonTest.staticVersionInfo,
    queryId = V1JsonTest.staticQueryProgress.id,
    adapterNodeId = V1JsonTest.staticNodeId,
    adapterNodeName = V1JsonTest.staticNode.name,
    status = ResultStatuses.IdAssigned,
    statusMessage = Some("test status message"),
    crcQueryInstanceId = Some(0),
    count = 0,
    resultType = ResultOutputType.PATIENT_COUNT_XML,
    breakdowns = None
  )

  val expectedErrorResult = ErrorResult(
    id = V1JsonTest.staticResultId,
    versionInfo = V1JsonTest.staticVersionInfo,
    queryId = V1JsonTest.staticQueryProgress.id,
    adapterNodeId = V1JsonTest.staticNodeId,
    adapterNodeName = V1JsonTest.staticNode.name,
    status = ResultStatuses.IdAssigned,
    statusMessage = Some("test ErrorFromCrc message"),
    JsonProblemDigest(TestProblem())
  )

  @Test
  def testResultAgainstV1Json():Unit = {
    V1JsonTest.testRoundTrip(jsonResultProgress, expectedResultProgress, Result.tryRead)
    V1JsonTest.testRoundTrip(jsonCrcResult, expectedCrcResult, Result.tryRead)
    V1JsonTest.testRoundTrip(jsonErrorResult, expectedErrorResult, Result.tryRead)
  }
}
