package net.shrine.protocol.version.v1

import net.shrine.problem.{JsonProblemDigest, TestProblem}
import net.shrine.protocol.ResultOutputType
import net.shrine.protocol.version.QueryId
import org.junit.Test

class UpdateResultTest {
  val jsonUpdateResultWithProgress = V1JsonTest.readJsonFile("/updateResultWithProgress.json")
  val jsonUpdateResultWithError = V1JsonTest.readJsonFile("/updateResultWithError.json")
  val jsonUpdateResultWithCrcResult = V1JsonTest.readJsonFile("/updateResultWithCrcResult.json")

  val expectedUpdateResultWithProgress = UpdateResultWithProgress(
    queryId = new QueryId(V1JsonTest.staticQueryId),
    adapterNodeKey = V1JsonTest.staticNode.key,
    status = V1JsonTest.staticResultProgress.status,
    statusMessage = Some("test status message")
  )
  val expectedUpdateResultWithError = UpdateResultWithError(
    queryId = new QueryId(V1JsonTest.staticQueryId),
    adapterNodeKey = V1JsonTest.staticNode.key,
    problem = JsonProblemDigest(TestProblem()),
    status = V1JsonTest.staticResultProgress.status,
    statusMessage = Some("test status message"),
    crcQueryInstanceId = Some(2)
  )
  val expectedUpdateResultWithCrcResult = UpdateResultWithCrcResult(
    queryId = new QueryId(V1JsonTest.staticQueryId),
    adapterNodeKey = V1JsonTest.staticNode.key,
    count = 1,
    crcQueryInstanceId = 2,
    resultType = ResultOutputType.PATIENT_COUNT_XML,
    status = V1JsonTest.staticResultProgress.status,
    statusMessage = Some("test status message")
  )

  @Test
  def testUpdateQueryResultAgainstV1Json():Unit = {
    V1JsonTest.testRoundTrip(jsonUpdateResultWithProgress, expectedUpdateResultWithProgress, UpdateResult.tryRead)
    V1JsonTest.testRoundTrip(jsonUpdateResultWithError, expectedUpdateResultWithError, UpdateResult.tryRead)
    V1JsonTest.testRoundTrip(jsonUpdateResultWithCrcResult, expectedUpdateResultWithCrcResult, UpdateResult.tryRead)
  }
}
