package net.shrine.protocol.version.v1

import net.shrine.crypto.NewTestKeyStore
import net.shrine.problem.{JsonProblemDigest, TestProblem}
import net.shrine.protocol.version.{DateStamp, EnvelopeContents, ItemVersion, JsonText, MomQueueName, NodeId, NodeKey, NodeName, ProtocolVersion, QueryId, QueryStatuses, ResearcherId, ResultId, ResultStatuses, TopicId, UserDomainName, UserName}
import net.shrine.util.ShouldMatchersForJUnit

import scala.io.Source
import scala.util.Try


object V1JsonTest extends ShouldMatchersForJUnit {
  val staticUserDomainName = new UserDomainName("testUserDomainName")
  val staticUserName = new UserName("testResearcherName")
  val staticQueryId = 0
  val staticNodeId = new NodeId(0)
  val staticResultId = new ResultId(0)
  val staticResearcherId = new ResearcherId(0)
  val staticTopicId = new TopicId(0)
  val staticDateStamp = new DateStamp(0)
  val staticVersionInfo = VersionInfo(
    protocolVersion = ProtocolVersion.current,
    itemVersion = ItemVersion.one,
    createDate = staticDateStamp,
    changeDate = staticDateStamp
  )
  val staticNode = new Node(
    id = staticNodeId,
    name = new NodeName("testNode"),
    key = new NodeKey("testNode"),
    userDomainName = new UserDomainName("testnode.domain"),
    adminEmail = "email@foo",
    versionInfo = staticVersionInfo,
    momQueueName = new MomQueueName("testMomQueueName")
  )
  val staticQueryProgress = Query.create(
    id = staticQueryId,
    versionInfo = staticVersionInfo,
    queryDefinitionXml = "queryXml",
    outputTypesXml = "queryOutputTypes",
    queryName = "queryName",
    nodeOfOriginId = staticNodeId,
    researcherId = staticResearcherId,
    topicId = staticTopicId,
    projectName = "projectName",
    certCollection = NewTestKeyStore.certCollection
  )
  val staticQueryError = QueryError(
    id = new QueryId(staticQueryId),
    versionInfo = staticVersionInfo,
    status = QueryStatuses.HubError,
    queryDefinitionXml = "queryXml",
    queryDefinitionSignature = staticQueryProgress.queryDefinitionSignature,
    outputTypesXml = "queryOutputTypes",
    queryName = "queryName",
    nodeOfOriginId = staticNodeId,
    researcherId = staticResearcherId,
    topicId = staticTopicId,
    projectName = "projectName",
    flagged = false,
    flaggedMessage = None,
    problemDigest = JsonProblemDigest(TestProblem())
  )
  val staticResearcher = Researcher(
    id = staticResearcherId,
    versionInfo = staticVersionInfo,
    userName = staticUserName,
    userDomainName = staticUserDomainName,
    nodeId = staticNodeId
  )
  val staticTopic = Topic(
    id = V1JsonTest.staticTopicId,
    versionInfo = V1JsonTest.staticVersionInfo,
    researcherId = V1JsonTest.staticResearcherId,
    name = "test RunQueryAtHub name",
    description = "just a test"
  )
  val staticResultProgress = ResultProgress(
    id = V1JsonTest.staticResultId,
    versionInfo = V1JsonTest.staticVersionInfo,
    queryId = V1JsonTest.staticQueryProgress.id,
    adapterNodeId = V1JsonTest.staticNodeId,
    adapterNodeName = V1JsonTest.staticNode.name,
    status = ResultStatuses.IdAssigned,
    statusMessage = Some("test status message"),
    crcQueryInstanceId = Some(0)
  )


  def testRoundTrip[A <: EnvelopeContents](json: String, obj: A, trans: JsonText => Try[A]) = {
    testObjToJson(json, obj)
    testJsonToObj(json, obj, trans)
  }

  def testJsonToObj[A <: EnvelopeContents](json: String, obj: A, trans: JsonText => Try[A]) = {
    assertResult(obj){ trans(new JsonText(json)).get }
  }

  def testObjToJson[A <: EnvelopeContents](json: String, obj: A) = {
    assertResult(json){ obj.asJsonText.underlying }
  }

  def readJsonFile(filename: String): String = {
    val source = Source.fromFile(getClass.getResource(filename).getFile)
    try source.mkString finally source.close()
  }
}