package net.shrine.source

import com.typesafe.config.{Config, ConfigFactory, ConfigParseOptions, ConfigResolveOptions}
import net.shrine.log.Log

import scala.util.control.NonFatal

/**
  * @author ty
  * @since  7/22/16
  */
object ConfigSource {

  //config names in order of precedence. qa takes top preference, reference.conf takes bottom
  val overrideConfigName: String = "override"
  val shrineConfigName: String = "shrine"
  val referenceConfigName: String = "reference"

  lazy val atomicConfig: AtomicConfigSource = new AtomicConfigSource ({
    //get each config level without resolving it

    val defaultOverrides = ConfigFactory.defaultOverrides()
    val qaConfig = ConfigFactory.parseResourcesAnySyntax(overrideConfigName)
    val shrineConfig = ConfigFactory.parseResourcesAnySyntax(shrineConfigName)
    val referenceConfig = ConfigFactory.parseResourcesAnySyntax(referenceConfigName)

    val finalConfig = defaultOverrides.
      withFallback(qaConfig).
      withFallback(shrineConfig).
      withFallback(referenceConfig).
      resolve()

    finalConfig
})

  def config: Config = {
    try atomicConfig.config
    catch {
      case NonFatal(x) =>
        Log.error(s"Could not load configuration from $shrineConfigName.conf and $overrideConfigName.conf due to ", x)
        throw x
    }
  }

  def configForBlock[T](key: String, value: AnyRef, origin: String)(block: => T): T = {
    atomicConfig.configForBlock(key, value, origin)(block)
  }

  def configForBlock[T](config: Config, origin: String)(block: => T): T = {
    atomicConfig.configForBlock(config, origin)(block)
  }
}

case class ConfigError(throwable: Throwable, path: String) extends Error {
  override def getMessage:String = s"Malformed config file, could not retrieve path '$path'"
}