package net.shrine.log

import java.io.{ByteArrayOutputStream, PrintStream}

import net.shrine.util.ShouldMatchersForJUnit
import org.junit.{After, AfterClass, BeforeClass, Test}
import org.junit.Assert.assertEquals

/**
  * @author david
  * @since 8/4/15
  */
class CustomPatternLayoutTest extends ShouldMatchersForJUnit {

  @After
  def afterEach(): Unit = {
    CustomPatternLayoutTest.out.reset()
  }

  val i2b2PasswordString = """<password is_token="false" token_ms_timeout="1800000">kapow</password>"""
  val expectedI2b2PasswordString = """<password is_token="false" token_ms_timeout="1800000">REDACTED</password>"""

  @Test
  def testCensorI2b2Message():Unit = {
    Log.debug(i2b2PasswordString)
    val result = CustomPatternLayoutTest.out.toString

    assert(result.contains(expectedI2b2PasswordString))
    assertEquals(CustomPatternLayoutTest.countMatches(result, expectedI2b2PasswordString), 1)
  }

  @Test
  def testCensorI2b2Throwable():Unit = {
    val exception = new IllegalArgumentException(i2b2PasswordString)

    Log.error("I2B2 sent back some strange xml", exception)
    val result = CustomPatternLayoutTest.out.toString

    assert(result.contains(expectedI2b2PasswordString))
    assertEquals(CustomPatternLayoutTest.countMatches(result, expectedI2b2PasswordString), 1)
  }

  @Test
  def testCensorI2b2ThrowableWithCause():Unit = {
    val exception = new IllegalArgumentException(i2b2PasswordString)
    exception.initCause(new RuntimeException(i2b2PasswordString))

    Log.error("I2B2 sent back some strange xml", exception)
    val result = CustomPatternLayoutTest.out.toString

    assertEquals(CustomPatternLayoutTest.countMatches(result, expectedI2b2PasswordString), 2)
  }
}

object CustomPatternLayoutTest {
  val out = new ByteArrayOutputStream
  val originalOut = System.out

  @BeforeClass
  def beforeClass(): Unit = {
    System.setOut(new PrintStream(out))
  }

  @AfterClass
  def afterClass(): Unit = {
    System.setOut(originalOut)
  }

  def countMatches(container: String, find: String): Int = {
    val tempString = container.replaceAll(find, "")
    (container.length - tempString.length) / find.length
  }
}