/*
  This is a fake qep/shrine-api/ service for local development and not intended for production in any way.
*/
const uniqueId = require("lodash.uniqueid");
const express = require("express");
const cors = require("cors");
const bodyParser = require("body-parser");
const bodyParserXml = require("body-parser-xml");
const devConfig = require("../dev.config");
const fs = require("fs");
const path = require("path");

bodyParserXml(bodyParser);
const app = express();
const port = process.env.PORT || devConfig.SERVICE_PORT;
const router = express.Router();
const isAuthorized = false;

const start = () => {
  // Configure Middleware.
  app.use(bodyParser.urlencoded({ extended: true }));
  app.use(bodyParser.json());
  app.use(
    bodyParser.xml({
      limit: "1GB", // Reject payload bigger than 1 MB
      xmlParseOptions: {
        normalize: true, // Trim whitespace inside text nodes
        normalizeTags: true, // Transform tags to lowercase
        explicitArray: false // Only put nodes in array if >1
      }
    })
  );
  app.use(cors());
  app.use("/", router);

  const handleI2B2Request = (req, res) => {
    const requestType = parseRequest(req);
    const fileName = getFilename(requestType);
    const file = path.join(__dirname, `/i2b2-xml/${fileName}`);
    const xml = fs.readFileSync(file);
    res.header("Content-Type", "text/xml").send(xml);
  };

  router.post("/shrine-api/i2b2/pmService", handleI2B2Request);

  router.post("/shrine-api/i2b2/ontologyService", handleI2B2Request);

  router.post("/shrine-api/i2b2/request", handleI2B2Request);

  router.get("/shrine-api/staticData/webClientConfig", function(req, res) {
    res.json({
      domain: "LOCAL DEV i2b2demo",
      name: "LOCAL DEV SHRINE"
    });
  });

  router.get("/shrine-api/qep/queryResultAuth", (req, res) => {
    const token = "eyJraWQiOiJWc";
    setTimeout(() => res.send(token)), 500;
  });

  app.get("/shrine-api/qep/sse/queryResultSet/eyJraWQiOiJWc", (req, res) => {
    const queryData = require("./data/queryData");
    // SSE setup
    res.writeHead(200, {
      "Content-Type": "text/event-stream",
      "Cache-Control": "no-cache",
      Connection: "keep-alive"
    });
    res.write("\n");

    // Send 'auth' Event.
    res.write("id: eyJraWQiOiJWetc\n");
    res.write("event: auth\n");
    res.write(
      `data: ${JSON.stringify({
        session: "valid"
      })}\n\n`
    );

    // Send query result. Every 8 seconds.
    setInterval(() => {
      res.write("id: eyJraWQiOiJWetc\n");
      res.write("event: queryResult-eyJraWQiOiJWetc\n");
      res.write(`data: ${JSON.stringify(queryData)}\n\n`);
    }, 8000);
  });

  router.get("/shrine-api/qep/queryResultPolling", (req, res) => {
    const { limit, sortBy, skip, networkId } = req.query;
    const selectedQuery = require("./data/selectedQuery.js");
    const allQueries = require("./data/allQueries.js");
    const rowCount = allQueries.length;
    const endIndex = skip + limit <= rowCount ? skip + limit : rowCount;

    setTimeout(
      () =>
        res.json({
          rowCount,
          rowOffset: skip,
          selectedQuery,
          isAllQueriesComplete: true,
          allQueries: allQueries.slice(skip, endIndex)
        }),
      1000
    );
  });

  router.get("/shrine-api/qep/networkId", (req, res) => {
    const service = require("./data/async-queries");
    const url = require("url");
    const data = url.parse(req.url, true).query;
    const result = service.getNetworkId(data.queryName);
    res.json(result);
  });

  //Start listening.
  app.listen(port);
  console.log("I2B2SERVER Mock Server started on port: " + port);
};

// Parse XML methods
const parseRequest = ({ body }) => {
  const requestTypes = [
    "i2b2:request",
    "ns2:request",
    "ns3:request",
    "ns6:request"
  ];
  const requestType = requestTypes.find(t => body[t] !== undefined);
  const request =
    body[requestType] && body[requestType].message_body
      ? body[requestType].message_body
      : null;
  return parseBodyRequest(request);
};

const parseBodyRequest = request => {
  const bodyTypes = [
    "pm:get_user_configuration",
    "ns4:get_categories",
    "ns4:get_schemes",
    "ns7:sheriff_header",
    "ns4:psmheader"
  ];

  let bodyType = bodyTypes.find(t => request[t] !== undefined);
  if (bodyType === "ns4:psmheader") {
    bodyType = request["ns4:psmheader"].request_type;
  }

  return bodyType;
};

const getFilename = value => {
  const fileMap = {
    "pm:get_user_configuration": "getUserAuth.xml",
    "ns4:get_categories": "GetCategories.xml",
    "ns4:get_schemes": "GetSchemes.xml",
    "ns7:sheriff_header": "getUserAuth.xml",
    CRC_QRY_getResultType: "getQRY_getResultType.xml",
    CRC_QRY_getQueryMasterList_fromUserId: "getQueryMasterList_fromUserId.xml"
  };

  return fileMap[value];
};

module.exports = start;
