/*
  babel-polyfill conflicts with prototype.js, install regenerator runtime to enable generators:
  https://github.com/facebook/regenerator/blob/master/packages/regenerator-runtime/README.md
*/
import 'regenerator-runtime/runtime';
import { setContext } from 'retrofitter';
import { getStore } from 'sandbox';
import { render } from 'sandbox';
import { queryNetworkIdReceived, queryStarted } from 'sandbox/actions';

const store = getStore();
(async () => {
  const context = await setContext(document.window, store);
  const queryTabsMount = document.getElementById('QUERY_TABS_MOUNT');
  const queryViewportMount = document.getElementById('QUERY_VIEWPORT_MOUNT');
  const queryExportMount = document.getElementById('QUERY_EXPORT_MOUNT');
  const queryHistoryMount = document.getElementById('QUERY_HISTORY_MOUNT');
  const queryHistoryOptionsMount = document.getElementById(
    'QUERY_HISTORY_OPTIONS_MOUNT',
  );
  const runQueryMount = document.getElementById('RUN_QUERY_MOUNT');
  const modalMount = document.getElementById('MODAL_MOUNT');
  const historyPagingMount = document.getElementById('HISTORY_PAGING_MOUNT');

  await render(store, {
    queryViewportMount,
    queryTabsMount,
    queryExportMount,
    queryHistoryMount,
    queryHistoryOptionsMount,
    runQueryMount,
    modalMount,
    historyPagingMount,
  });

  await context.i2b2.hive.mySplitter.dragged();
  jQuery('#screenQueryData').css('visibility', 'visible');

  if (
    ['http://localhost:8888/', 'http://127.0.0.1:8888/'].includes(document.URL)
  ) {
    const networkId = '123456';
    const name = 'local dev: testing...1, 2, 3';
    store.dispatch(queryStarted({ name: 'query-started' }));
    setTimeout(
      () => store.dispatch(queryNetworkIdReceived({ networkId, name })),
      1000,
    );
  }
})();
