import {
  queryStarted,
  clearQuery,
  cancelQueryPoll,
} from 'sandbox/actions';

export const queryRunDecorator = (context) => {
  const { store } = context;

  return (inQueryName, options) => {
    context.$('.sdxDefaultQM').css('background-color', 'white');
    context.i2b2.events.clearQuery.fire();
    store.dispatch(queryStarted({ name: inQueryName, data: options }));
    // make sure name is not blank
    if (inQueryName.blank()) {
      alert('Cannot run query with without providing a name!');
      return;
    }
    // Query Parameters
    var query_definition = context.i2b2.CRC.ctrlr.QT._getQueryXML(inQueryName);
    var params = {
      result_wait_time: context.i2b2.CRC.view.QT.params.queryTimeout,
      psm_query_definition: query_definition.queryXML
    }
    if (context.i2b2.h.isSHRINE()) {
      var topicSELECT = context.prototype$('queryTopicSelect');
      if (topicSELECT) {
        if (topicSELECT.selectedIndex == null || topicSELECT.selectedIndex == 0) {
          alert("Please select a Topic to run the query.");
          return false;
        }
        params.shrine_topic = "<shrine><queryTopicID>" + topicSELECT.options[topicSELECT.selectedIndex].value + "</queryTopicID><queryTopicName>" + topicSELECT.options[topicSELECT.selectedIndex].label + "</queryTopicName></shrine>";
      }
    }

    let i = 0;
    let result_output = `<result_output priority_index="${i}" name="patient_count_xml" />\n`;
    for (var name in options) {
      if (name) {
        i++;
        result_output += '<result_output priority_index="' + i + '" name="' + name.substring(4).toLowerCase() + '"/>\n';
      }
    }

    params.psm_result_output = '<result_output_list>' + result_output + '</result_output_list>\n';
    context.i2b2.CRC.ctrlr.currentQueryStatus = new context.i2b2.CRC.ctrlr.QueryStatus(context.prototype$('infoQueryStatusText'));
    context.i2b2.CRC.ctrlr.currentQueryStatus.startQuery(inQueryName, params);
  }
}

export const queryClearDecorator = (context) => {
  const { store } = context;
  const doQueryClear = context.i2b2.CRC.ctrlr.QT.doQueryClear;
  return clearStatus => {
    doQueryClear.apply(context.i2b2.CRC.ctrlr.QT, []);
    if (clearStatus === true) {
      context.$('.sdxDefaultQM').css('background-color', 'white');
      store.dispatch(cancelQueryPoll());
      store.dispatch(clearQuery());
      store
    }
  }
}

export const queryLoadDecorator = (context) => {
  const doQueryLoad = context.i2b2.CRC.ctrlr.QT.doQueryLoad;
  return qmId => {
    doQueryLoad.apply(context.i2b2.CRC.ctrlr.QT, qmId);
  }
}


