import { queryNetworkIdReceived } from 'sandbox/actions';

export const startQueryDecorator = (context) => {
  const { store } = context;

  function startQuery(queryName, ajaxParams) {
    const self = context.i2b2.CRC.ctrlr.currentQueryStatus;
    if (self.isQueryRunning()) return;
    self.isQueryRunning(true);
    self.name = queryName; // for consistency with i2b2.
    self.params = ajaxParams;
    context.i2b2.CRC.view.status.showDisplay();
    context.i2b2.CRC.ctrlr.currentQueryResults = new context.i2b2.CRC.ctrlr.QueryResults(queryName);

    // callback processor to run the query from definition
    this.callbackQueryDef = new context.global.i2b2_scopedCallback();
    this.callbackQueryDef.scope = this;
    this.callbackQueryDef.callback = (results) => {
      if (!context.i2b2.CRC.ctrlr.currentQueryStatus) return;
      const networkId = results.refXML.getElementsByTagName('query_master_id')[0].firstChild.nodeValue;
      store.dispatch(queryNetworkIdReceived({ networkId }));
      clearQuery();
    };

    const clearQuery = () => {
      self.isQueryRunning(false);
      self.currentQueryStatus = false;
      context.$('#dialogQryRunResultType input[type="checkbox"]')
        .each((a, b) => {
          b.checked = b.disabled;
        });
    };

    const params = context.i2b2.CRC.ctrlr.currentQueryStatus.params;
    context.i2b2.CRC.ajax.runQueryInstance_fromQueryDefinition('CRC:QueryTool', ajaxParams, this.callbackQueryDef);
  }
  return startQuery;
};

export const isQueryRunningDecorator = (running = false) =>
  (state) => {
    if (state !== undefined) running = state;
    return running;
  };

export const refreshStatusDecorator = () => () => { /* empty method for compatibility with i2b2 */ };

