/**
 * This module modifies the global i2b2 object in order to bootstrap the SHRINE sandbox.
 * to allow the shrine plugin.
 */
import ContextWrapper from './common/ContextWrapper';
import {
  startQueryDecorator,
  isQueryRunningDecorator,
  refreshStatusDecorator,
} from './decorators/CRC.ctrlr.QueryStatus';
import {
  queryRunDecorator,
  queryClearDecorator,
} from './decorators/CRC.ctrlr.QryTool';
import ResizeHeightDecorator from './decorators/CRC.view.Status';
import getChildRecordsDecorator from './decorators/CRC.sdx.QM';
import { NoSynonymTermInfoDecorator } from './decorators/i2b2.messages';
import { historyRefreshDecorator } from './decorators';


class ShrineBootstrapper extends ContextWrapper {
  bootstrapTo(context, store) {
    this.store = store;
    this.setContext(context)
      .decorateI2B2();
  }

  decorateI2B2() {
    this.i2b2.CRC.ctrlr.QT._queryRun = queryRunDecorator(this);
    this.i2b2.CRC.view.status.showDisplay = () => {
      /*  empty method to not break referenes from i2b2 */
    };
    this.i2b2.CRC.ctrlr.QT.doQueryClear = queryClearDecorator(this);
    this.i2b2.h.LoadingMask.show = () => {
      /*  empty emthod to not break references from i2b2 */
    };
    this.i2b2.CRC.ctrlr.QueryStatus.prototype.startQuery = startQueryDecorator(this);
    this.i2b2.CRC.ctrlr.QueryStatus.prototype.isQueryRunning = isQueryRunningDecorator();
    this.i2b2.CRC.ctrlr.QueryStatus.prototype.refreshStatus = refreshStatusDecorator();
    this.i2b2.CRC.view.status.ResizeHeight = ResizeHeightDecorator(this);
    this.i2b2.sdx.TypeControllers.QM.getChildRecords = getChildRecordsDecorator(this);
    NoSynonymTermInfoDecorator(this)(false);
    const roles = this.i2b2.PM.model.userRoles.filter(r => r !== 'MANAGER');
    this.i2b2.PM.model.userRoles = roles;
    this.i2b2.CRC.ctrlr.history.Refresh = historyRefreshDecorator(this);
    return this;
  }
}

// -- singleton --//
export default new ShrineBootstrapper();
