import ContextWrapper from './common/ContextWrapper';
import { ErrorDialogue } from './components';
import {
  i2b2NetworkError,
} from 'sandbox/actions';
import './shrinePlugin.scss';

/**
 * This module contains any SHRINE Webclient logic that cannot be ported into the Sandbox.
 */
export class ShrinePlugin extends ContextWrapper {
  store;
  init(context, store) {
    this.store = store;
    return new Promise((resolve) => {
      this.setContext(context);
      const PLUGIN_ID = 'shrinePlugin';
      const CellViewController = this.global.i2b2Base_cellViewController;
      const { CustomEvent } = this.YAHOO.util;
      // TODO: not sure if this custom event is necessary see decorators.
      // -- Global Events -- //
      this.i2b2.events.clearQuery = new CustomEvent('clearQuery', this.i2b2);
      // -- Global Methods -- //
      this.i2b2.SHRINE.plugin = new CellViewController(this.i2b2.SHRINE, PLUGIN_ID);
      this.i2b2.SHRINE.plugin.errorDetail = this.errorDetail.bind(this);
      this.i2b2.SHRINE.plugin.dispatchNetworkError = this.dispatchNetworkError.bind(this);
      this.i2b2.events.afterLogin.subscribe(() => resolve());
    });
  }
  errorDetail = (data) => {
    const emailTo = this.i2b2.hive.cfg.webclientConfig.siteAdminEmail;
    this.$('#pluginErrorDetail').remove();
    this.$('body').append(this.$(ErrorDialogue({ data, emailTo })));
    const { SimpleDialog } = this.YAHOO.widget;
    const pluginErrorDetail = new SimpleDialog('pluginErrorDetail', {
      width: '820px',
      fixedcenter: true,
      constraintoviewport: true,
      modal: true,
      zindex: 700,
      buttons: [{
        text: 'Done',
        handler: function () { this.cancel(); }, // must use function instead of arrow.
        isDefault: true,
      }],
      validate: () => true,
    });
    this.prototype$('pluginErrorDetail').show();
    pluginErrorDetail.render(document.body);
    pluginErrorDetail.center();
    pluginErrorDetail.show();
  }

  dispatchNetworkError = (errorData, url) => {
    const { result_status = { status: null } } = errorData;
    const defaultErrorMsg = 'An unknown error has occurred.  If this is a recurring issue please contact your local SHRINE administrator.';
    const defaultStatus = 'ERROR';

    this.store.dispatch(i2b2NetworkError({
      url,
      status: result_status.status._type || defaultStatus,
      statusText: result_status.status.__text || defaultErrorMsg,
    }));
  }
}

// SINGLETON
export default new ShrinePlugin();
