import React from 'react';
import { render as reactRender } from 'react-dom';
import { Provider } from 'react-redux';
import { StatusAndGraphTabContainer } from 'sandbox/components/StatusAndGraphTabs';
import { StatusAndGraphViewContainer } from 'sandbox/components/StatusAndGraphViews';
import { QueryExport } from 'sandbox/components/QueryExport';
import { QueryHistoryContainer, Paging } from 'sandbox/components/QueryHistory';
import { ModalContainer } from 'sandbox/components/Modal';
import { RunQuery } from 'sandbox/components/RunQuery';

import './App.scss';

export const render = (store, mountConfig) => {
  return new Promise(resolve => {
    const {
      queryTabsMount,
      queryViewportMount,
      queryExportMount,
      queryHistoryMount,
      runQueryMount,
      modalMount,
      historyPagingMount,
    } = mountConfig;
    reactRender(
      <Provider store={store}>
        <StatusAndGraphTabContainer />
      </Provider>,
      queryTabsMount,
    );
    reactRender(
      <Provider store={store}>
        <StatusAndGraphViewContainer />
      </Provider>,
      queryViewportMount,
    );
    reactRender(
      <Provider store={store}>
        <QueryExport label="Export to CSV" />
      </Provider>,
      queryExportMount,
    );
    reactRender(
      <Provider store={store}>
        <QueryHistoryContainer />
      </Provider>,
      queryHistoryMount,
    );
    reactRender(
      <Provider store={store}>
        <RunQuery />
      </Provider>,
      runQueryMount,
    );
    reactRender(
      <Provider store={store}>
        <ModalContainer />
      </Provider>,
      modalMount,
    );
    reactRender(
      <Provider store={store}>
        <Paging />
      </Provider>,
      historyPagingMount,
    );
    resolve();
  });
};
