import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { BreakdownResultType } from 'sandbox/types';
import range from 'lodash.range';
import { MarkerList } from './MarkerList';
import { BarList } from './BarList';

export class Graph extends Component {
  static propTypes = {
    title: PropTypes.string.isRequired,
    results: PropTypes.arrayOf(BreakdownResultType),
  }

  getMarkerHeight = ({ length }) => {
    const barHeight = 18;
    return { height: `${barHeight * length}px` };
  };
  getUpperBound = (maxValue, multiple = 100) => {
    const upperBoundPlusRemainder = maxValue + multiple;
    return (maxValue <= multiple) ?
      multiple : upperBoundPlusRemainder - (upperBoundPlusRemainder % multiple);
  };

  getHighestResult = results =>
    results.reduce((acc, cur) => {
      const { value: curMax } = cur;
      const { value: accMax } = acc;
      return (curMax < accMax) ? acc : cur;
    });
  getRange = (upperBound, count = 10) => {
    const increment = upperBound / count;
    return range(0, upperBound + increment, increment);
  };

  makeGetWidth = upperBound => value => (
    value <= 0 ? 0 : (value / upperBound) * 100
  );

  renderEmptyResults(title) {
    return (
      <div className="Graph">
        <div className="title">
          {title}
        </div>
        <div className="no-results">
          Results are unavailable for this breakdown.
        </div>
      </div>
    );
  }

  renderResults(results, title) {
    const { value: highestValue } = this.getHighestResult(results);
    const upperBound = this.getUpperBound(highestValue);
    return (
      <div className="Graph">
        <div className="title">
          {title}
        </div>
        <MarkerList
          range={this.getRange(upperBound)}
          css={this.getMarkerHeight(results)}
        />
        <BarList
          results={results}
          getWidth={this.makeGetWidth(upperBound)}
        />
      </div>
    );
  }

  render() {
    const { results, title } = this.props;
    return (
      !results || !results.length
        ? this.renderEmptyResults(title)
        : this.renderResults(results, title)
    );
  }
}
