import React from 'react';
import { shallow } from 'enzyme';
import { shallowToJson } from 'enzyme-to-json';
import { Graph } from '../Graph';

describe('The Graph Component', () => {

  const props = {
    results: [{ dataKey: 'dataKey', value: 10 }],
    title: 'Graph Component'
  };

  const graphInstance = new Graph(props);

  it('Should render correctly', () => {
    const output = shallow(
      <Graph
        {...props}
      />,
    );
    expect(shallowToJson(output)).toMatchSnapshot();
  });

  it('getMarkerHeight should calculate the height in px of graph markers based on # of results', () => {
    const expected =  { height: '72px' };
    const actual = graphInstance.getMarkerHeight({ length: 4 });
    expect(actual).toEqual(expected);
  });

  it('getUpperBound should calculate an upper bound for the graph that is larger than the highest result, to a specified multiple', () => {
    const maxResultValue = 270;
    const multiple = 100;
    const expected = 300;
    const actual = graphInstance.getUpperBound(maxResultValue, multiple);
    expect(actual).toEqual(expected);
  });

  it('getHighestResult should return the breakdown result with the highest value', () => {
    const results = [{
      value: 17,
    }, {
      value: 46,
    }, {
      value: 15,
    }];
    const expected = { value: 46 };
    const actual = graphInstance.getHighestResult(results);
    expect(actual).toEqual(expected);
  });

  it('getRange should return an array of values that between 0 and the highest value', () => {
    const upperBound = 800;
    const count = 4;
    const expected = [0, 200, 400, 600, 800];
    const actual = graphInstance.getRange(upperBound, count);
    expect(actual).toEqual(expected);
  });

  it('makeGetWidth should return a method that divides a given value by an upper bound as a value between 0 and 100', () => {
    const upperBound = 800;
    const getWidth = graphInstance.makeGetWidth(upperBound);
    const expected = 50;
    const actual = getWidth(400);
    expect(actual).toBe(expected);
  });
});
