import React, { Component } from 'react';
import PropTypes from 'prop-types';

import { getProp } from 'sandbox/services';
import './FlagQuery.scss';
import { FlaggingForm } from './FlaggingForm';

export class FlagQuery extends Component {
  static propTypes = {
    onClose: PropTypes.func.isRequired,
    id: PropTypes.string.isRequired,
    flaggedMessage: PropTypes.string,
    flagged: PropTypes.bool,
  }

  static defaultProps = {
    flaggedMessage: '',
    flagged: false,
  };

  static getDerivedStateFromProps(nextProps, state) {
    const { flaggedMessage } = nextProps;
    const { text } = state;

    if (text === null) {
      return { text: flaggedMessage };
    }

    return null;
  }

  state = {
    text: null,
    unflag: false,
  }

  onFlaggingMessageChange = (event) => {
    event.stopPropagation();
    const text = event.target.value;

    this.setState({ text });
  }

  toggleUnflag = () => {
    const unflag = !this.state.unflag;
    this.setState({ unflag });
  };

  handleSubmit = (event) => {
    event.preventDefault();
    const { text: message, unflag } = this.state;
    const { id: queryId, onClose } = this.props;

    onClose(true);
    if (unflag) {
      getProp('i2b2.CRC.ctrlr.history.Unflag')
        .apply(getProp('i2b2.CRC.ctrlr.history'), [{ queryId }]);
    } else {
      getProp('i2b2.CRC.ctrlr.history.Flag')
        .apply(getProp('i2b2.CRC.ctrlr.history'), [{ queryId, message }]);
    }
  }

  render() {
    const {
      onClose,
      id,
      flagged,
      flaggedMessage,
    } = this.props;
    const { text, unflag } = this.state;
    const disableSubmit = (!text || !text.length) && !unflag;
    const { queryFlaggingInstructions, flaggingPlaceholderText } = getProp('i2b2.hive.cfg.webclientConfig');

    return (
      <FlaggingForm
        onClose={onClose}
        id={id}
        flagged={flagged}
        flaggedMessage={flaggedMessage}
        text={text}
        unflag={unflag}
        queryFlaggingInstructions={queryFlaggingInstructions}
        flaggingPlaceholderText={flaggingPlaceholderText}
        handleSubmit={this.handleSubmit}
        toggleUnflag={this.toggleUnflag}
        disableSubmit={disableSubmit}
        onFlaggingMessageChange={this.onFlaggingMessageChange}
      />
    );
  }
}
