import React from 'react';
import PropTypes from 'prop-types';
import sanitizeHtml from 'sanitize-html';

export const FlaggingForm = (props) => {
  const {
    onClose,
    id,
    flagged,
    flaggedMessage,
    text,
    unflag,
    handleSubmit,
    queryFlaggingInstructions,
    flaggingPlaceholderText,
    onFlaggingMessageChange,
    disableSubmit,
    toggleUnflag,
  } = props;

  return (
    <form
      className="FlagQuery"
      onSubmit={handleSubmit}
    >
      <div className="hd">Flag Query</div>
      <div className="bd">
        {(queryFlaggingInstructions &&
          <p
            dangerouslySetInnerHTML={{ __html: sanitizeHtml(queryFlaggingInstructions) }}
          />
        )}
        <span className="label">
          <strong>Query ID:</strong>
        </span>
        <span className="label">
          <strong>{id}</strong>
        </span>
        <div>
          <div className="label">
            <br />
            <strong>Flagged Message:</strong>
          </div>
          <textarea
            readOnly={unflag}
            placeholder={flaggingPlaceholderText || 'Enter a flagging message here'}
            onChange={onFlaggingMessageChange}
            value={text}
          />

        </div>
        <div>
          <span className="label">
            {(flagged &&
              <span>
                Select to unflag query
                <input
                  name="unflag"
                  type="checkbox"
                  checked={unflag}
                  onChange={toggleUnflag}
                />
              </span>
            )}
          </span>
        </div>
      </div>
      <div className="ft">
        <span className="button-group">
          <span className="yui-button yui-push-button default">
            <span className="first-child">
              <button
                type="submit"
                className={disableSubmit ? 'disabled' : 'submit'}
                disabled={disableSubmit}
              >
                {!flagged && 'Flag Query'}
                {flagged && unflag && 'Unflag Query'}
                {flagged && !unflag && 'Update Query'}
              </button>
            </span>
          </span>
          <span className="yui-button yui-push-button">
            <span className="first-child">
              <button
                type="button"
                onClick={onClose}
              >Cancel
              </button>
            </span>
          </span>
        </span>
      </div>
    </form>
  );
};

FlaggingForm.propTypes = {
  handleSubmit: PropTypes.func.isRequired,
  queryFlaggingInstructions: PropTypes.string.isRequired,
  flaggingPlaceholderText: PropTypes.string.isRequired,
  flaggedMessage: PropTypes.string.isRequired,
  onClose: PropTypes.func.isRequired,
  flagged: PropTypes.bool.isRequired,
  unflag: PropTypes.bool.isRequired,
  id: PropTypes.string.isRequired,
  text: PropTypes.string.isRequired,
  disableSubmit: PropTypes.bool.isRequired,
  toggleUnflag: PropTypes.func.isRequired,
  onFlaggingMessageChange: PropTypes.func.isRequired,
};
