import React from 'react';
import PropTypes from 'prop-types';

import './Modal.scss';
import { RenameQuery } from './RenameQuery';
import { FlagQuery } from './FlagQuery';

export const Modal = ({ onClose, visible, data }) => {
  const className = visible ? 'Modal active' : 'Modal';

  let ModalContent;
  switch (data.type) {
    case 'rename':
      ModalContent = (
        <RenameQuery
          onClose={onClose}
          {...data}
        />
      );
      break;
    case 'flag': {
      ModalContent = (
        <FlagQuery
          onClose={onClose}
          {...data}
        />
      );
      break;
    }
    default: ModalContent = null;
  }

  return (
    <div className={className} role="dialog" aria-hidden="true">
      {ModalContent}
    </div>
  );
};

Modal.propTypes = {
  data: PropTypes.shape({}).isRequired,
  onClose: PropTypes.func.isRequired,
  visible: PropTypes.bool.isRequired,
};
