import { connect } from 'react-redux';
import React, { Component } from 'react';
import PropTypes from 'prop-types';

import { hideModal, fetchQueryPending } from 'sandbox/actions';
import { Modal } from './Modal';

class ModalContainer extends Component {

  static propTypes = {
    showModal: PropTypes.bool,
    dispatch: PropTypes.func.isRequired,
    modalData: PropTypes.shape({}).isRequired,
  };

  static defaultProps = {
    showModal: false,
  }

  close = (waitingOnI2B2Fetch = false) => {
    const { dispatch } = this.props;

    dispatch(hideModal());

    /*
     It is not ideal to have the UI explicitly set pending state on a REST call, however there
     is latency in communication between the i2b2 XML API, so show the loader immediatley in
     this instance.
    */
    if (waitingOnI2B2Fetch) {
      dispatch(fetchQueryPending());
    }
  }

  render() {
    const { showModal, modalData } = this.props;

    return (
      <Modal
        onClose={this.close}
        visible={showModal}
        data={modalData}
      />
    );
  }
}

export const mapStateToProps = ({ modal }) => ({ ...modal });
export default connect(mapStateToProps)(ModalContainer);
