import React, { Component } from 'react';
import PropTypes from 'prop-types';

import './RenameQuery.scss';
import { getProp } from 'sandbox/services';

export class RenameQuery extends Component {
  static propTypes = {
    onClose: PropTypes.func.isRequired,
    name: PropTypes.string.isRequired,
    id: PropTypes.string.isRequired,
  }


  static getDerivedStateFromProps(nextProps, state) {
    const { name } = nextProps;
    const { name: currentName } = state;

    if (currentName === null) {
      return { name };
    }

    return null;
  }

  state = { name: null };

  onNameChange = (event) => {
    event.stopPropagation();
    const name = event.target.value;

    this.setState({ name });
  }

  handleSubmit = (event) => {
    event.preventDefault();

    const { name } = this.state;
    const { id, onClose, name: oldName } = this.props;

    onClose(true);
    getProp('i2b2.CRC.ctrlr.history.queryRename')
      .apply(getProp('i2b2.CRC.ctrlr.history'), [id, name, oldName]);
  }

  render() {
    const { name = '' } = this.state;
    const { onClose } = this.props;
    const disableSubmit = name === null || name.length < 1;

    return (
      <form
        className="RenameQuery"
        onSubmit={this.handleSubmit}
      >
        <div className="hd">Edit Query Name</div>
        <div className="bd">
          <span className="label">
            Query Name:
          </span>
          <input onChange={this.onNameChange} value={name} />
        </div>
        <div className="ft">
          <span className="button-group">
            <span className="yui-button yui-push-button default">
              <span className="first-child">
                <button
                  type="submit"
                  className={disableSubmit ? 'disabled' : 'submit'}
                  disabled={disableSubmit}
                >Edit Query Name
                </button>
              </span>
            </span>
            <span className="yui-button yui-push-button">
              <span className="first-child">
                <button
                  type="button"
                  onClick={onClose}
                >
                  Cancel
                </button>
              </span>
            </span>
          </span>
        </div>
      </form>
    );
  }
}
