import React from 'react';
import { shallow } from 'enzyme';
import { shallowToJson } from 'enzyme-to-json';

import { FlaggingForm } from '../FlaggingForm';

describe('FlaggingForm', () => {
  const props = {
    onClose: jest.fn(),
    id: '12345',
    flagged: false,
    flaggedMessage: 'mock flagged message',
    text: 'mock message text',
    unflag: false,
    queryFlaggingInstructions: 'mock query flagging instructions',
    handleSubmit: jest.fn(),
    toggleUnflag: jest.fn(),
    disableSubmit: false,
    onFlaggingMessageChange: jest.fn(),
  };

  const output = shallow(
    <FlaggingForm {...props} />,
  );
 
  it('Should render correctly', () => {
    expect(shallowToJson(output)).toMatchSnapshot();
  });

  it('Should not allow a user to unflag a query when a query is not flagged', () => {
    expect(output.find({ type: 'checkbox' }).length).toBe(0);
  });

  it('Should allow a user to edit or create a flagging message if unflag is false', () => {
    expect(output.find('textarea').length).toBe(1);
  });

  it('Should allow a user to unflag a query if the query is flagged', () => {
    const unflaggableProps = { ...props, flagged: true };
    const unflaggableOutput = shallow(
      <FlaggingForm {...unflaggableProps} />,
    );
    
    unflaggableOutput.find({ type: 'checkbox' }).simulate('change');
    expect(unflaggableProps.toggleUnflag).toHaveBeenCalled();
  });
});
