import React from 'react';
import { shallow } from 'enzyme';
import { shallowToJson } from 'enzyme-to-json';

import PagingButton from '../PagingButton';
import firtstImg from '../img/first.jpg';
import firtstInactiveImg from '../img/firstInactive.jpg';

describe('PagingButton', () => {
  const props = {
    isEnabled: true,
    img: firtstImg,
    disabledImg: firtstInactiveImg,
    onClick: jest.fn(),
    type: 'mock paging button',
  };

  it('Should render correctly', () => {
    const output = shallow(<PagingButton {...props} />);
    expect(shallowToJson(output)).toMatchSnapshot();
  });

  it('should call onClick when enabled', () => {
    const output = shallow(<PagingButton {...props} />);
    expect(props.isEnabled).toBe(true);
    output.simulate('click');
    expect(props.onClick).toHaveBeenCalled();
  });

  it('Should ignore onClick when disabled', () => {
    const newProps = { ...props, isEnabled: false, onClick: jest.fn() };
    const output = shallow(<PagingButton {...newProps} />);
    expect(newProps.isEnabled).toBe(false);
    output.simulate('click');
    expect(newProps.onClick).not.toHaveBeenCalled();
  });
});
