import { connect } from 'react-redux';
import React, { Component } from 'react';
import PropTypes from 'prop-types';

import './Paging.scss';
import { History } from 'sandbox/types';
import { fetchQueryResults } from 'sandbox/actions';
import firstImg from './img/first.jpg';
import firstInactiveImg from './img/firstInactive.jpg';
import prevImg from './img/prev.jpg';
import prevInactiveImg from './img/prevInactive.jpg';
import nextImg from './img/next.jpg';
import nextInactiveImg from './img/nextInactive.jpg';
import lastImg from './img/last.jpg';
import lastInactiveImg from './img/lastInactive.jpg';
import PagingButton from './PagingButton';

export class WrappedPaging extends Component {
  static propTypes = {
    history: PropTypes.instanceOf(History).isRequired,
    dispatch: PropTypes.func.isRequired,
  };

  fetchFirstPage = () => {
    const { limit } = this.props.history;
    this.props.dispatch(
      fetchQueryResults({
        rowOffset: 0,
        limit,
      }),
    );
  };

  fetchPreviousPage = () => {
    const { rowOffset, limit } = this.props.history;
    const currentOffset = rowOffset - limit;
    if (currentOffset >= 0) {
      this.props.dispatch(
        fetchQueryResults({
          rowOffset: currentOffset,
          limit,
        }),
      );
    }
  };

  fetchNextPage = () => {
    const { rowCount, rowOffset, limit } = this.props.history;
    const currentOffset = rowOffset + limit;
    if (currentOffset < rowCount) {
      this.props.dispatch(
        fetchQueryResults({
          rowOffset: currentOffset,
          limit,
        }),
      );
    }
  };

  fetchLastPage = () => {
    const { rowCount, limit } = this.props.history;
    const rowOffset =
      rowCount % limit > 0 ? rowCount - (rowCount % limit) : rowCount - limit;

    this.props.dispatch(
      fetchQueryResults({
        rowOffset,
        limit,
      }),
    );
  };

  render() {
    const { rowCount, limit, rowOffset } = this.props.history;
    const canGoBack = rowOffset > 0;
    const canGoForward = rowOffset + limit < rowCount;

    return (
      <div className="Paging">
        <PagingButton
          type="first"
          onClick={this.fetchFirstPage}
          img={firstImg}
          disabledImg={firstInactiveImg}
          isEnabled={canGoBack}
        />
        <PagingButton
          type="previous"
          onClick={this.fetchPreviousPage}
          img={prevImg}
          disabledImg={prevInactiveImg}
          isEnabled={canGoBack}
        />
        <PagingButton
          type="next"
          onClick={this.fetchNextPage}
          img={nextImg}
          disabledImg={nextInactiveImg}
          isEnabled={canGoForward}
        />
        <PagingButton
          type="last"
          onClick={this.fetchLastPage}
          img={lastImg}
          disabledImg={lastInactiveImg}
          isEnabled={canGoForward}
        />
      </div>
    );
  }
}

export const mapStateToProps = ({ history }) => ({ history });
export default connect(mapStateToProps)(WrappedPaging);
