import { connect } from 'react-redux';
import React, { Component } from 'react';
import PropTypes from 'prop-types';

import { HistoryType } from 'sandbox/types';
import {
  flagQuery,
  unflagQuery,
  renameQuery,
  fetchQueryResults,
  cancelQueryPoll,
  clearQuery,
  queryNetworkIdReceived,
} from 'sandbox/actions';
import { getProp } from 'sandbox/services';
import { FetchError } from 'sandbox/components/FetchError';
import { QueryHistoryTable } from './QueryHistoryTable';

class QueryHistoryContainer extends Component {
  static propTypes = {
    history: HistoryType.isRequired,
    dispatch: PropTypes.func.isRequired,
  };

  componentDidMount() {
    this.props.dispatch(fetchQueryResults());
  }

  showFlagModal = modalData => {
    this.props.dispatch(flagQuery({ modalData }));
  };

  showUnflagModal = modalData => {
    this.props.dispatch(unflagQuery({ modalData }));
  };

  showRenameModal = modalData => {
    this.props.dispatch(renameQuery({ modalData }));
  };

  sortQueryHistory = sortBy => {
    const { dispatch } = this.props;

    dispatch(fetchQueryResults({ sortBy }));
  };

  loadQueryResultFromHistory = ({ id: networkId, name: queryName }) => {
    this.props.dispatch(cancelQueryPoll());
    this.props.dispatch(clearQuery());
    // -- use apply to preserve 'this' scope of global i2b2 object -- //
    getProp('i2b2.CRC.ctrlr.QT.doQueryLoad').apply(
      getProp('i2b2.CRC.ctrlr.QT'),
      [networkId],
    );

    this.props.dispatch(
      queryNetworkIdReceived({
        networkId,
        name: queryName,
      }),
    );
  };

  render() {
    const { history } = this.props;
    return (
      <div>
        {history.error ? (
          <FetchError url={history.url} errorMsg={history.errorMsg} />
        ) : (
          <span>
            <QueryHistoryTable
              history={history}
              selectedResultId={history.selectedResultId}
              resultSelected={this.loadQueryResultFromHistory}
              onScrollReachedTop={this.fetchPreviousQueryHistory}
              onScrollReachedBottom={this.fetchNextQueryHistory}
              onHeaderClicked={this.sortQueryHistory}
              elementId="crcHistoryData"
              onFlagClicked={this.showFlagModal}
              onRenameClicked={this.showRenameModal}
            />
          </span>
        )}
      </div>
    );
  }
}

export const mapStateToProps = ({ history }) => ({ history });
export default connect(mapStateToProps)(QueryHistoryContainer);
