import React, { Component } from 'react';
import PropTypes from 'prop-types';

import './QueryHistoryTable.scss';
import { QueryHistoryTableHeader } from './QueryHistoryTableHeader';
import { QueryTableLoader } from './QueryTableLoader';
import { QueryHistoryTableBody } from './QueryHistoryTableBody';
import { Tooltip } from './Tooltip';
import { getOrElse } from 'sandbox/services';

export class QueryHistoryTable extends Component {
  static propTypes = {
    history: PropTypes.shape({}).isRequired,
    resultSelected: PropTypes.func.isRequired,
    selectedResultId: PropTypes.string.isRequired,
    onScrollReachedTop: PropTypes.func.isRequired,
    onScrollReachedBottom: PropTypes.func.isRequired,
    elementId: PropTypes.string.isRequired,
    onHeaderClicked: PropTypes.func.isRequired,
    onFlagClicked: PropTypes.func.isRequired,
    onUnflagClicked: PropTypes.func.isRequired,
    onRenameClicked: PropTypes.func.isRequired,
  };

  state = { selectedHeaderId: 'dateCreated' };
  updateSelectedHeaderId = selectedHeaderId =>
    this.setState({ selectedHeaderId });
  handleHeaderClicked = data => {
    const { id, sort } = data;
    const { onHeaderClicked } = this.props;

    this.updateSelectedHeaderId(id);
    onHeaderClicked(`${id}.${sort}`);
  };

  render() {
    const {
      history,
      resultSelected,
      selectedResultId,
      onScrollReachedBottom,
      onScrollReachedTop,
      elementId,
      onFlagClicked,
      onRenameClicked,
    } = this.props;
    const { selectedHeaderId } = this.state;
    const tableClassName = history.isFetching
      ? 'table table-hover table-mc-light-blue loading'
      : 'table table-hover table-mc-light-blue';
    const { flaggingIconInstructions } = getOrElse(
      'i2b2.hive.cfg.webclientConfig',
      'Flag a Query',
    );

    return (
      <div className="QueryHistoryTable">
        {history.isFetching && <QueryTableLoader />}
        <table className={tableClassName}>
          <thead>
            <tr>
              <th>
                <i className="fa fa-circle-o" aria-hidden="true" />
              </th>
              <QueryHistoryTableHeader
                id="queryName"
                label="Query Name"
                selected={selectedHeaderId === 'queryName'}
                onClick={this.handleHeaderClicked}
                isFetching={history.isFetching}
              />
              <QueryHistoryTableHeader
                id="dateCreated"
                label="Run Date"
                selected={selectedHeaderId === 'dateCreated'}
                onClick={this.handleHeaderClicked}
                isFetching={history.isFetching}
              />
              <th>
                <i className="fa fa-flag" aria-hidden="true" />
                {!history.isFetching && (
                  <Tooltip text={flaggingIconInstructions} />
                )}
              </th>
              <th>
                <i className="fa fa-pencil" aria-hidden="true" />
              </th>
            </tr>
          </thead>
          <QueryHistoryTableBody
            results={history.results}
            resultSelected={resultSelected}
            selectedResultId={selectedResultId}
            onFlagClicked={onFlagClicked}
            onRenameClicked={onRenameClicked}
          />
        </table>
      </div>
    );
  }
}
