import React from 'react';
import PropTypes from 'prop-types';
import { List } from 'immutable';

import { QueryHistoryTableRow } from './QueryHistoryTableRow';

export const QueryHistoryTableBody = ({
  results,
  resultSelected,
  selectedResultId,
  onFlagClicked,
  onRenameClicked,
}) => (
  <tbody>
    {results.map(result => (
      <QueryHistoryTableRow
        key={result.networkId}
        name={result.queryName}
        flagged={result.flag && result.flag.flagged}
        flaggedMessage={result.flag && result.flag.flagMessage}
        id={result.networkId}
        runDate={result.dateCreated}
        onSelected={resultSelected}
        selected={selectedResultId === result.networkId}
        observed={result.observed}
        onFlagClicked={onFlagClicked}
        onRenameClicked={onRenameClicked}
        isError={result.isQueryError || result.isResultsError}
        status={result.status}
      />
    ))}
  </tbody>
);

QueryHistoryTableBody.propTypes = {
  resultSelected: PropTypes.func.isRequired,
  selectedResultId: PropTypes.string.isRequired,
  results: PropTypes.instanceOf(List).isRequired,
  onFlagClicked: PropTypes.func.isRequired,
  onUnflagClicked: PropTypes.func.isRequired,
  onRenameClicked: PropTypes.func.isRequired,
};
