import React, { Component } from 'react';
import PropTypes from 'prop-types';

import { timestamp } from 'sandbox/converters';

export class QueryHistoryTableRow extends Component {
  static propTypes = {
    isError: PropTypes.bool.isRequired,
    name: PropTypes.string.isRequired,
    runDate: PropTypes.string.isRequired,
    id: PropTypes.string.isRequired,
    flagged: PropTypes.bool,
    flaggedMessage: PropTypes.string,
    selected: PropTypes.bool,
    observed: PropTypes.bool,
    onSelected: PropTypes.func.isRequired,
    onFlagClicked: PropTypes.func.isRequired,
    onRenameClicked: PropTypes.func.isRequired,
  }

  static defaultProps = {
    selected: false,
    observed: false,
    flagged: false,
    flaggedMessage: '',
  };

  handleClick = () => {
    const { id, name, onSelected } = this.props;

    onSelected({ id, name });
  }

  handleFlagClicked = (event) => {
    event.stopPropagation();
    const {
      onFlagClicked,
      name,
      id,
      flaggedMessage = '',
      flagged = false,
    } = this.props;

    onFlagClicked({
      name,
      id,
      flaggedMessage,
      flagged,
    });
  }

  handleRenameClicked = (event) => {
    event.stopPropagation();
    const { onRenameClicked, name, id } = this.props;

    onRenameClicked({ id, name });
  }

  render() {
    const {
      name,
      flagged,
      runDate,
      selected,
      observed,
      isError,
    } = this.props;
    const selectedClass = selected ? 'selected' : '';
    const observedClass = observed ? 'observed' : '';
    const flagClassNames = flagged ? 'fa fa-flag hover-controls flagged' : 'fa fa-flag-o hover-controls';
    const statusClassName = isError ? 'fa fa-circle error' : 'fa fa-circle';

    return (
      <tr
        className={`${observedClass} ${selectedClass}`}
        onClick={this.handleClick}
      >
        <td>
          <i className={statusClassName} aria-hidden="true" />
        </td>
        <td>{name}</td>
        <td>{timestamp(runDate)}</td>
        <td>
          <i
            className={flagClassNames}
            aria-hidden="true"
            onClick={this.handleFlagClicked}
          />
        </td>
        <td>
          <i
            className="fa fa-pencil hover-controls"
            aria-hidden="true"
            onClick={this.handleRenameClicked}
          />
        </td>
      </tr>
    );
  }
}
