import React from 'react';
import { shallow } from 'enzyme';
import { shallowToJson } from 'enzyme-to-json';

import { QueryHistoryTableRow } from '../QueryHistoryTableRow';

describe('QueryHistoryTableRow', () => {
  const props = {
    status: 'Mock Query Status',
    name: 'Mock History Table Row Name',
    runDate: 'Mock run date',
    id: 'Mock Id',
    flagged: false,
    selected: false,
    onSelected: jest.fn(),
    onFlagClicked: jest.fn(),
    onUnflagClicked: jest.fn(),
    onRenameClicked: jest.fn(),
  };

  it('Should render correctly', () => {
    const output = shallow(
      <QueryHistoryTableRow {...props} />,
    );
    expect(shallowToJson(output)).toMatchSnapshot();
  });

  it('should handle a row click', () => {
    const output = shallow(
      <QueryHistoryTableRow {...props} />,
    );
    output.simulate('click');
    expect(props.onSelected).toHaveBeenCalled();
  });

  it('should handle a rename icon click', () => {
    const output = shallow(
      <QueryHistoryTableRow {...props} />,
    );
    output.find('.fa-pencil').at(0).simulate('click', { stopPropagation: jest.fn() });
    expect(props.onRenameClicked).toHaveBeenCalled();
  });

  it('should handle a flag icon click', () => {
    const output = shallow(
      <QueryHistoryTableRow {...props} />,
    );
    output.find('.fa-flag-o').at(0).simulate('click', { stopPropagation: jest.fn() });
    expect(props.onFlagClicked).toHaveBeenCalled();
  });

  it('should handle a unflag icon click', () => {
    const output = shallow(
      <QueryHistoryTableRow {...{ ...props, flagged: true }} />,
    );
    output.find('.fa-flag').at(0).simulate('click', { stopPropagation: jest.fn() });
    expect(props.onFlagClicked).toHaveBeenCalled();
  });
});
