import React from 'react';
import { shallow } from 'enzyme';

import { RunQueryButton } from '../RunQueryButton';

describe('RunQueryButton', () => {
  it('Should contain the busy CSS classname if Query is running', () => {
    const output = shallow(<RunQueryButton
      className={'busy'}
      handleClick={jest.fn()}
    />,
    );
    expect(output.find('.busy').length).toBe(1);
    expect(output.find('.RunQuery').length).toBe(0);
  });

  it('Should default RunQuery CSS classname if Query is not running', () => {
    const output = shallow(<RunQueryButton
      className={'RunQuery'}
      handleClick={jest.fn()}
    />,
    );
    expect(output.find('.busy').length).toBe(0);
    expect(output.find('.RunQuery').length).toBe(1);
  });

  it('Should handle a click event', () => {
    const handleClick = jest.fn();
    const output = shallow(<RunQueryButton
      className={'RunQuery'}
      handleClick={handleClick}
    />,
    );
    output.simulate('click');
    expect(handleClick).toHaveBeenCalled();
  })
});

