import React, { Component, Fragment } from 'react';

import { QueryType } from 'sandbox/types';
import { dateTime } from 'sandbox/converters';
import './SelectedQuery.scss';
import { InstitutionResultStatusList, QueryStatus } from '../Statuses';
import { getProp } from 'sandbox/services';

class SelectedQuery extends Component {

  static propTypes = {
    query: QueryType.isRequired,
  }

  errorDetailPopUp = (result) => {
    getProp('i2b2.SHRINE.plugin.errorDetail')(result);
  }

  render() {
    const { query } = this.props;
    return (
      <div className="SelectedQuery">
        {query.networkId === null &&
          query.institutionResults.length === 0 &&
          <div className="idle">{query.statusMsg}</div>
        }
        <div className="details">
          {query.name !== null && <div className="name">Status of Query {query.name}</div>}
          <div>
            {(query.name !== null &&
              <Fragment>
                <QueryStatus
                  query={query}
                  handleErrorStatusClicked={this.errorDetailPopUp}
                />
              </Fragment>
            )}
          </div>
          <div className="updated">
            {query.updated !== null && `Last Updated on: ${dateTime(query.updated)}`}
          </div>
        </div>
        {
          query.networkId !== null &&
          query.institutionResults.length === 0 &&
          query.isComplete === false &&
          <div className="no-results">{query.statusMsg}</div>
        }
        <div className="resultsTitle">
          {query.institutionResults.length > 0 && 'at Each Institution: '}
        </div>
        <InstitutionResultStatusList
          errorDetailPopUp={this.errorDetailPopUp}
          institutionResults={query.institutionResults}
        />
      </div >
    );
  }
}

export default SelectedQuery;
