import React from 'react';
import { shallow } from 'enzyme';
import { shallowToJson } from 'enzyme-to-json';

import SelectedQuery from './../SelectedQuery';

describe('Selected Query', () => {

  const query = {
    name: 'Mock Query',
    networkId: '12345',
    updated: 1001,
    statusMsg: 'Mock status message',
    internalStatus: 'Mock internal status',
    status: 'Complete',
    error: false,
    errorMsg: null,
    url: 'mock url',
    isRunning: false,
    isQueryError: false,
    isResultsError: false,
    institutionResults: [{
      institutionName: 'Mock Institution',
      count: 100,
      status: 'Mock status',
      internalStatus: 'Mock Internal Status',
      statusMessage: 'Mock node status',
      error: false,
      problemDigest: null,
    }],
  };

  it('should render correctly', () => {
    const output = shallow(
      <SelectedQuery
        query={query}
      />,
    );

    expect(shallowToJson(output)).toMatchSnapshot();
  });

  it('Should render an error if isQueryError is set to true', () => {

    const props = {
      ...query,
      isQueryError: true,
      status: 'Mock Query Error',
      intenalStatus: 'Mock internal Query Error Status',
    };
    const output = shallow(
      <SelectedQuery
        query={props}
      />,
    );
    expect(output.html().includes('QueryError')).toBe(true);
    expect(output.html().includes('QueryNonError')).toBe(false);
    expect(output.html().includes('Completed')).toBe(false);
  });
  
  it('Should render green Completed text if isQueryError is false and isResults error is true', () => {
    const props = {
      ...query,
      isResultsError: true,
      status: 'Completed',
      intenalStatus: 'Completed',
    };
    const output = shallow(
      <SelectedQuery
        query={props}
      />,
    );
    expect(output.html().includes('QueryError')).toBe(false);
    expect(output.html().includes('QueryNonError')).toBe(true);
    expect(output.html().includes('Completed')).toBe(true);
  });
});
