import React from 'react';
import PropTypes from 'prop-types';

import { Status } from './Status';
import { ClickableStatus } from './ClickableStatus';
import { count as countConverter } from 'sandbox/converters';
import { InstitutionResultType } from 'sandbox/types';

export const InstitutionResultStatus = (props) => {
  const { handleStatusClicked, result } = props;
  const { internalStatus } = result;

  let resultType;
  //todo clean this up in SHRINE-3670 - use the values from Shrine's enum supplied via a json field
  switch (internalStatus.toUpperCase()) {
    case 'RESULT FROM CRC':
    case 'FINISHED':
      resultType = (
        <Status
          statusText={countConverter(result.count)}
          className="PatientCount"
        />
      );
      break;
    case 'ID ASSIGNED':
      resultType = (
        <Status
          statusText="Processing at Hub"
          className="SentToSite"  //todo fix with SHRINE-3670
        />
      );
      break;
    case 'SENT TO ADAPTER':
      resultType = (
        <Status
          statusText="Sent to Site"
          className="SentToSite"
        />
      );
      break;
    case 'SUBMITTED TO CRC':
    case 'RECEIVED BY ADAPTER':
    case 'READY TO SUBMIT':
    case 'SUBMITTED TO CRC':
      resultType = (
        <Status
          statusText="Processing at Site"
          className="ProcessingAtSite"
        />
      );
      break;
    case 'ERROR IN SHRINE':
    case 'UKNOWN WHILE QUEUED BY CRC':
    case 'ERROR FROM CRC':
    case 'UNKNOWN FINAL':
    case 'ERROR':
    case 'PROCESSING':
    case 'RUNNING':
    case 'QUEUED':
    case 'INCOMPLETE':
    case 'HELD':
    case 'SMALL_QUEUE':
    case 'TIMEDOUT':
    case 'MEDIUM_QUEUE':
    case 'LARGE_QUEUE':
    case 'NO_MORE_QUEUE':
    case 'MEDIUM_QUEUE_RUNNING':
    case 'LARGE_QUEUE_RUNNING':
    case 'HUB_WILL_SUBMI':
      resultType = (
        <ClickableStatus
          statusText="Site Error"
          className="SiteError"
          data={result}
          handleStatusClicked={handleStatusClicked}
        />
      );
      break;
    default:
      resultType = (
        <Status
          statusText="Delayed At Site"
          className="DelayedAtSite"
        />
      );
  }


  return (
    <div className="QueryCount">
      <div className="institutionName"> {result.institutionName}</div>
      {resultType}
    </div>
  );
};

InstitutionResultStatus.propTypes = {
  result: InstitutionResultType.isRequired,
  handleStatusClicked: PropTypes.func.isRequired,
};
