import React from 'react';
import { shallow } from 'enzyme';
import { shallowToJson } from 'enzyme-to-json';
import { List } from 'immutable';

import { withInfiniteScroll } from '../withInfiniteScroll';

describe('withInfiniteScroll', () => {
  const MockComponent = () => <span id="mockCompoment">Mock Component</span>;
  const getMoreItems = jest.fn();
  const getPreviousItems = jest.fn();
  const MockComponentWithInfiniteScroll = withInfiniteScroll(MockComponent);

  it('Should render the content of the wrapped component', () => {
    const output = shallow(
      <MockComponentWithInfiniteScroll
        onScrollReachedBottom={getMoreItems}
        onScrollReachedTop={getPreviousItems}
        elementId="mockComponent"
      />,
    );
    expect(shallowToJson(output)).toMatchSnapshot();
  });

  it('Should not call the onScrollReachedBottom method on mount', () => {
    const mockComponentInstance = new MockComponentWithInfiniteScroll({
      elementId: 'mockComponent',
      onScrollReachedBottom: getMoreItems,
      onScrollReachedTop: getPreviousItems,
    });
    expect(getMoreItems).not.toBeCalled();
    expect(getPreviousItems).not.toBeCalled();
  });

  it('Should not call the onScrollReachedBottom method if the scroll has not reached the bottom', () => {
    const mockComponentInstance = new MockComponentWithInfiniteScroll({
      elementId: 'mockComponent',
      onScrollReachedBottom: getMoreItems,
      onScrollReachedTop: getPreviousItems,
    });
    mockComponentInstance.element = {
      scrollHeight: 100,
      offsetHeight: 50,
      scrollTop: 30,
    };
    mockComponentInstance.handleScroll();
    expect(getMoreItems).not.toBeCalled();
    expect(getPreviousItems).not.toBeCalled();
  });

  it('Should call the onScrollReachedBottom method if the scroll has reached the bottom', () => {
    const mockComponentInstance = new MockComponentWithInfiniteScroll({
      elementId: 'mockComponent',
      onScrollReachedBottom: getMoreItems,
      onScrollReachedTop: getPreviousItems,
    });
    //scrollHeight, offsetHeight, scrollTop
    mockComponentInstance.element = {
      scrollHeight: 100,
      offsetHeight: 52,
      scrollTop: 50,
    };
    mockComponentInstance.previousScrollDistance =
      mockComponentInstance.element.scrollHeight -
      mockComponentInstance.element.offsetHeight;
    mockComponentInstance.handleScroll();
    expect(getMoreItems).toBeCalled();
    expect(getPreviousItems).not.toBeCalled();
  });
  it('Should call the onScrollReachedTop method if the scroll has reached the top', () => {
    const onScrollReachedBottom = jest.fn();
    const mockComponentInstance = new MockComponentWithInfiniteScroll({
      elementId: 'mockComponent',
      onScrollReachedBottom,
      onScrollReachedTop: getPreviousItems,
    });
    mockComponentInstance.element = {
      scrollHeight: 100,
      offsetHeight: 50,
      scrollTop: 0,
    };
    mockComponentInstance.handleScroll();
    expect(onScrollReachedBottom).not.toBeCalled();
    expect(getPreviousItems).toBeCalled();
  });
});
