import React, { Component } from 'react';
import PropTypes from 'prop-types';

export const withInfiniteScroll = WrappedComponent =>
  class WithInfiniteScroll extends Component {
    static propTypes = {
      onScrollReachedBottom: PropTypes.func.isRequired,
      onScrollReachedTop: PropTypes.func.isRequired,
      elementId: PropTypes.string.isRequired,
    };

    componentDidMount() {
      this.element = document.getElementById(this.props.elementId);
      if (this.element) {
        this.previousScrollDistance =
          this.element.scrollHeight - this.element.offsetHeight;
        this.element.addEventListener('scroll', this.handleScroll, false);
      }
    }

    componentWillUnmount() {
      if (this.element) {
        this.element.removeEventListener('scroll', this.handleScroll, false);
        this.element = null;
      }
    }

    element = null;
    handleScroll = () => {
      const { scrollHeight, offsetHeight, scrollTop } = this.element;

      if (scrollTop <= 0) {
        this.props.onScrollReachedTop();
        return;
      }

      const scrollOffset = 2;
      const totalScrollDistance = scrollHeight - offsetHeight;
      const scrollWasTriggeredByResize =
        totalScrollDistance !== this.previousScrollDistance;
      const scrollReachedBottom =
        Math.ceil(scrollTop) ===
          Math.ceil(totalScrollDistance + scrollOffset) &&
        !scrollWasTriggeredByResize;
      this.previousScrollDistance = totalScrollDistance;

      if (scrollReachedBottom) {
        this.props.onScrollReachedBottom();
      }
    };

    render() {
      return <WrappedComponent {...this.props} />;
    }
  };
