import { tabReducer } from '../tabReducer';

describe('tabReducer', () => {
  it('Should return a default state for unhandled cases', () => {
    const action = { type: 'ACTION_DOES_NOT_EXIST' };
    const expected = { selectedTab: 'QUERY_STATUS' };
    const actual = tabReducer(undefined, action);
    expect(actual).toEqual(expected);
  });

  it('Should set the selectedTab property to value of the payload', () => {
    const action = { type: 'SELECT_TAB', payload: 'TheTabIWant' };
    const expected = { selectedTab: 'TheTabIWant' };
    const actual = tabReducer({}, action);
    expect(actual).toEqual(expected);
  });

  it('Should set the selectedTab to QueryStatus when a Network ID is received', () => {
    const action = { type: 'QUERY_NETWORK_ID_RECEIVED' };
    const expected = { selectedTab: 'QUERY_STATUS' };
    const actual = tabReducer({}, action);
    expect(actual).toEqual(expected);
  });

  it('Should set the selectedTab to QueryStatus when a Query is Started', () => {
    const action = { type: 'QUERY_STARTED' };
    const expected = { selectedTab: 'QUERY_STATUS' };
    const actual = tabReducer({}, action);
    expect(actual).toEqual(expected);
  });
});
