import { List } from 'immutable';
import { defaultState } from 'sandbox/defaultState';
import {
  FETCH_QUERY_RESULTS,
  FETCH_QUERY_RESULTS_PENDING,
  FETCH_QUERY_RESULTS_FULFILLED,
  QUERY_HISTORY_UPDATE,
  FETCH_QUERY_RESULTS_REJECTED,
  SELECT_QUERY_HISTORY_RESULT,
  CLEAR_QUERY,
  QUERY_STARTED,
  QUERY_NETWORK_ID_RECEIVED,
} from 'sandbox/actions';

const { history } = defaultState;
export const queryHistoryReducer = (state = history, action) => {
  switch (action.type) {
    case FETCH_QUERY_RESULTS: {
      if (action.payload === undefined) return state;
      const {
        limit = 0,
        sortBy = state.sortBy,
        rowOffset = state.rowOffset,
      } = action.payload;
      return limit > 0
        ? state.merge({ limit, sortBy, rowOffset })
        : state.merge({ sortBy, rowOffset });
    }
    case FETCH_QUERY_RESULTS_PENDING: {
      return state.merge({
        error: false,
        errorMsg: null,
        isFetching: true,
      });
    }
    case QUERY_HISTORY_UPDATE:
    case FETCH_QUERY_RESULTS_FULFILLED: {
      const {
        allQueries,
        rowCount,
        selectedResultId = state.selectedResultId,
      } = action.payload;
      return state.merge({
        results: new List(allQueries),
        rowCount,
        error: false,
        errorMsg: null,
        isFetching: false,
        selectedResultId,
      });
    }
    case FETCH_QUERY_RESULTS_REJECTED: {
      const { status, statusText, url } = action.errorData;
      return state.merge({
        error: true,
        url,
        errorMsg: `${status} ${statusText}`,
        isFetching: false,
      });
    }
    case QUERY_NETWORK_ID_RECEIVED:
    case SELECT_QUERY_HISTORY_RESULT: {
      const { networkId } = action.payload;
      return state.merge({ selectedResultId: networkId });
    }
    case CLEAR_QUERY:
    case QUERY_STARTED: {
      return state.merge({ selectedResultId: null });
    }
    default: {
      return state;
    }
  }
};
