import { call, takeLatest, put, select } from 'redux-saga/effects';
import fetch from 'isomorphic-fetch';

import {
  FETCH_QUERY,
  fetchQueryRejected,
  fetchQueryFulfilled,
  fetchQueryPending,
} from 'sandbox/actions';
import { getBaseUrl, getAuth } from 'sandbox/services';

import { selectLimit, selectSortBy, selectRowOffset } from './selectors';

export function* doQueryFetch(action) {
  const { networkId, dataVersion } = action.payload;
  let timeoutSeconds = '';
  if (dataVersion === -1) {
    timeoutSeconds = '&timeoutSeconds=0';
  }

  try {
    yield put(fetchQueryPending());
    const limit = yield select(selectLimit);
    const sortBy = yield select(selectSortBy);
    const rowOffset = yield select(selectRowOffset);
    const url = `${getBaseUrl()}qep/queryResultPolling?networkId=${networkId}&limit=${limit}&sortBy=${sortBy}&skip=${rowOffset}${timeoutSeconds}`;
    const headers = { Authorization: `Basic  ${getAuth()}` };
    const response = yield call(fetch, url, { headers });
    if (response.ok === true) {
      yield put(fetchQueryFulfilled(response));
    } else {
      yield put(
        fetchQueryRejected({
          response,
          networkId,
        }),
      );
    }
  } finally {
    const msg = `Fetch for networkId: ${networkId || 'none provided'} closed`;
    yield msg;
  }
}

export function* queryFetchSaga() {
  yield takeLatest(FETCH_QUERY, doQueryFetch);
}
