import { delay } from 'redux-saga';
import { take, put, takeLatest, fork, apply } from 'redux-saga/effects';

import {
  QUERY_NETWORK_ID_RECEIVED,
  FETCH_QUERY_FULFILLED,
  FETCH_QUERY_REJECTED,
  RESTART_QUERY_POLL,
  CANCEL_QUERY_POLL,
  restartQueryPoll,
  fetchQuery,
  queryUpdate,
  queryError,
  queryTimeout,
  queryHistoryUpdate,
} from 'sandbox/actions';

export function* queryFetchFulfilledSaga() {
  const { payload } = yield take(FETCH_QUERY_FULFILLED);
  const queryData = yield apply(payload, payload.json);
  const {
    rowCount,
    rowOffset,
    selectedQuery = null,
    allQueries = [],
    isAllQueriesComplete,
  } = queryData;

  if (selectedQuery !== null) {
    const { isComplete, query, dataVersion } = selectedQuery;
    const { networkId } = query;

    yield put(
      queryHistoryUpdate({
        allQueries,
        rowCount,
        rowOffset,
        selectedResultId: networkId,
      }),
    );
    yield put(queryUpdate(selectedQuery));

    if (isComplete === false || !isAllQueriesComplete) {
      yield put(restartQueryPoll({ networkId, dataVersion }));
    }
  } else {
    yield put(
      queryHistoryUpdate({
        allQueries,
        rowCount,
        rowOffset,
      }),
    );
  }
}

export function* queryFetchRejectedSaga() {
  const { errorData } = yield take(FETCH_QUERY_REJECTED);
  const { networkId = null } = errorData;
  const { status, statusText, url } = errorData.response;
  if (status === 429 || status === 503) {
    yield put(
      queryTimeout({
        status,
        statusText,
        url,
        networkId,
      }),
    );
    yield delay(15000);
    yield put(restartQueryPoll({ networkId }));
  } else {
    yield put(
      queryError({
        status,
        statusText,
        url,
        networkId,
      }),
    );
  }
}

export function* queryPollSaga(action) {
  const { networkId, dataVersion = -1 } = action.payload;
  try {
    yield fork(queryFetchRejectedSaga);
    yield fork(queryFetchFulfilledSaga);
    yield put(fetchQuery({ networkId, dataVersion }));
  } finally {
    console.log(
      `Polling thread for networkId: ${networkId} ${dataVersion} cancelled`,
    );
  }
}

export function* startQuerySaga() {
  while (true) {
    const queryPollThread = yield takeLatest(
      [QUERY_NETWORK_ID_RECEIVED, RESTART_QUERY_POLL],
      queryPollSaga,
    );

    yield take(CANCEL_QUERY_POLL);
    if (queryPollThread.isRunning()) {
      yield queryPollThread.cancel();
    }
  }
}
