import { call, takeLatest, put, apply, select } from 'redux-saga/effects';
import fetch from 'isomorphic-fetch';

import {
  FETCH_QUERY_RESULTS,
  fetchQueryResultsPending,
  fetchQueryResultsFulfilled,
  fetchQueryResultsRejected,
} from 'sandbox/actions';
import { getBaseUrl, getAuth } from 'sandbox/services';
import { selectLimit, selectSortBy, selectRowOffset } from './selectors';

export function* doQueriesFetch() {
  try {
    yield put(fetchQueryResultsPending());
    const limit = yield select(selectLimit);
    const sortBy = yield select(selectSortBy);
    const rowOffset = yield select(selectRowOffset);
    const url = `${getBaseUrl()}qep/queryResultPolling?&limit=${limit}&sortBy=${sortBy}&skip=${rowOffset}`;
    const headers = { Authorization: `Basic ${getAuth()}` };
    const response = yield call(fetch, url, { headers });
    if (response.ok === true) {
      const queryData = yield apply(response, response.json);
      const { rowCount, rowOffset: offset, allQueries = [] } = queryData;

      yield put(
        fetchQueryResultsFulfilled({ allQueries, rowCount, rowOffset: offset }),
      );
    } else {
      const { status, statusText, url: errorUrl } = response;
      yield put(
        fetchQueryResultsRejected({
          status,
          statusText,
          url: errorUrl,
        }),
      );
    }
  } finally {
    const msg = 'Fetch of Query History cancelled';
    yield msg;
  }
}

export function* queryResultsFetchSaga() {
  yield takeLatest(FETCH_QUERY_RESULTS, doQueriesFetch);
}
