import { makeGetAuth } from '../getAuth';


describe('Make Get Auth', () => {


  it('should create a method that returns null if window.storage does not exist', () => {
    const getAuth = makeGetAuth();
    const expected = null;
    const actual = getAuth();
    expect(actual).toEqual(expected);
  });

  it('makeGetAuth should create a method that securely retrieves a user auth from HTML5 storage cache to make it accessible for the session and deletes it from the cache after it has been retrieved.', () => {
    let mockAuth = 'auth1234';
    const mockStorage = { // window.storage mock implements getItem and removeItem.
      getItem: jest.fn(() => mockAuth),
      removeItem: jest.fn(() => {
        mockAuth = undefined;
      }),
    };

    const expected = mockAuth;
    const getAuth = makeGetAuth(mockStorage);
    const actual = getAuth();
    expect(actual).toEqual(expected);
    expect(mockAuth).toBeUndefined();
  });
});