import { getOrElse, getProp } from '../getOrElse';

describe('nodePathOption', () => {
  const root = {
    mockProp1: {
      mockProp2: 'mock prop 2',
    },
  };

  it('Should return a nested property if it exists', () => {
    const expected = 'mock prop 2';
    const actual = getOrElse('mockProp1.mockProp2', null, root);
    expect(actual).toBe(expected);
  });

  it('Should return the supplied default value if the property does not exist', () => {
    const expected = 'does not exist';
    const actual = getOrElse('propDoesNotExist.doesNotExist.doesNotExist', 'does not exist', root);
    expect(actual).toBe(expected);
  });


  it('Should operate on the DOM window by default and return undefined as the default property', () => {
    const expected = undefined;
    const actual = getOrElse('propertyDoesNotExist.nestedDoesNotExist');
    expect(actual).toBe(expected);
  });
});

describe('getProp', () => {
  const root = {
    mockProp1: {
      mockProp2: 'mock prop 2',
    },
  };

  it('Should throw a console error if a property does not exist', () => {
    const mockGlobalProperty = 'mockGlobal.property.does.not.exist';
    const expected = new Error(`${mockGlobalProperty} is undefined`);
    try {
      getProp('mockGlobal.property.does.not.exist', root);
    } catch (actual) {
      expect(actual).toEqual(expected);
    }
  });

  it('Should successfully access an existing property', () => {
    const expected = 'mock prop 2';
    const actual = getProp('mockProp1.mockProp2', root);
    expect(actual).toEqual(expected);
  })
});
