//
//   The legacy stack of YUI tools and I2B2 Client Code silently swallows any console errors.
//   This class is intended for accessing globally scoped i2b2 properties in the Legacy Web Client and for providing
//   the developer with a way to set an alternative default to an undefined property or forcce the browser to 
//   display errors:


//
//  getOrElse is intended for cases where a developer wants to provide a default value in case the
//  property being accessed is undefined
//
export const getOrElse = (path, none = undefined, root = window, token = '.') => {
  const prop = path.split(token).reduce((currentPosition, nextSegment) => (
    currentPosition !== undefined && currentPosition[nextSegment] !== undefined
      ? currentPosition[nextSegment]
      : undefined
  ), root);
  return prop !== undefined ? prop : none;
};

//
//   getProp is a convenience wrapper for getOrElse cases where a developer does not want to provide a default value for
//   a global property or method that does not exist and wants to have a console error instead.
//
export const getProp = (path, root = window) => {
  const option = getOrElse(path, undefined, root);
  if (option === undefined) {
    throw new Error(`${path} is undefined`);
  }
  return option;
};
