import { makeAscendingSortComparer } from './makeAscendingSortComparer';

export const mapInstitutionResultsToBreakdowns = (institutionResults) => {
  const filteredResults = institutionResults.filter((result) => {
    const { internalStatus, breakdowns = [] } = result;
    return (
      ['FINISHED', 'RESULT FROM CRC']
        .includes(internalStatus.toUpperCase()) &&
      breakdowns.length > 0
    );
  }).reduce((accum, { institutionName, breakdowns }) => {
    const descriptionSelector = ({ description = '' }) => description;
    const descriptionSorter = makeAscendingSortComparer(descriptionSelector);
    const sortedBreakdowns = breakdowns.map(({ results, resultType = { } }) => {
      const { i2b2Options = { description: '' } } = resultType;
      const dataKeySelector = ({ dataKey = '' }) => dataKey;
      const dataKeySorter = makeAscendingSortComparer(dataKeySelector);
      const sortedResults = results.sort(dataKeySorter);
      return { description: i2b2Options.description, results: sortedResults };
    }).sort(descriptionSorter);
    return { ...accum, [institutionName]: sortedBreakdowns };
  }, { });
  return filteredResults;
};

