export const DEFAULT_VALUE = 'unavailable';
export const NO_RESULTS_VALUE = -1;
export const EXPORT_FILENAME = 'export.csv';
export const countValueConverter = ({ count = DEFAULT_VALUE } = {}) => (count === NO_RESULTS_VALUE ? 0 : count);
export const resultValueConverter = ({ value = DEFAULT_VALUE } = {}) => (value === NO_RESULTS_VALUE ? 0 : value);

export const buildBreakdownSchema = ({ institutionResults, breakdowns }) => (
  institutionResults.reduce((schema, { institutionName }) => {
    if (breakdowns[institutionName] !== undefined) {
      const breakdown = breakdowns[institutionName];
      breakdown.forEach(({ description, results }) => {
        if (schema[description] === undefined) {
          schema[description] = { dataKeys: new Set() };
        }
        results.forEach(({ dataKey }) => {
          schema[description].dataKeys.add(dataKey);
        });
      });
    }
    return schema;
  }, {})
);

const findResultByDataKey = (breakdown, breakdownType, dataKey) => {
  if (breakdown !== undefined) {
    const { results = [] } = breakdown.find(({ description }) => description === breakdownType);
    const result = results.find(_ => _.dataKey === dataKey);
    return result;
  }
  return {};
};

export const mapQueryToCSV = (query) => {
  const { institutionResults, breakdowns } = query;
  const line1 = `SHRINE QUERY RESULTS (OBFUSCATED PATIENT COUNTS),, ${institutionResults.map(({ institutionName }) => institutionName).join(',')}`;
  const line2 = `\nAll Patients,,${institutionResults.map(countValueConverter).join(',')}`;
  const schema = buildBreakdownSchema(query);
  let breakdownRows = '';
  Object.entries(schema).forEach(([breakdownType, { dataKeys }]) => {
    dataKeys.forEach((dataKey) => {
      const rowTitle = `${breakdownType.trim()} ${dataKey}`;
      const values = institutionResults.reduce((line, { institutionName }) => {
        const result = findResultByDataKey(breakdowns[institutionName], breakdownType, dataKey);
        return `${line},${resultValueConverter(result)}`;
      }, '');
      breakdownRows = `${breakdownRows},\n${rowTitle},${values}`;
    });
  });
  return `${line1}${line2}${breakdownRows}`;
};
