package net.shrine.hub.data.client

import cats.effect.IO
import net.shrine.protocol.version.v1.{Network, Node}
import net.shrine.protocol.version.{NodeId, NodeKey}
import org.http4s.headers.Authorization
import org.http4s.{BasicCredentials, Headers, Method, Request, Uri}

object HubServiceRequests {

  def pingRequest(hubServiceUri: Uri): Request[IO] = Request(
    method = Method.GET,
    uri = hubServiceUri.withPath(s"${hubServiceUri.path}/ping")
  )

  def getNetworkRequest(hubServiceUri: Uri): Request[IO] = Request(
    method = Method.GET,
    uri = hubServiceUri.withPath(s"${hubServiceUri.path}/network")
  )

  def putNetworkRequest(hubServiceUri: Uri,network:Network): Request[IO] = Request(
    method = Method.PUT,
    uri = hubServiceUri.withPath(s"${hubServiceUri.path}/network")
  ).withEntity(network.asJsonText.underlying)

  def getNodeRequest(hubServiceUri: Uri,nodeKey:NodeKey): Request[IO] = Request(
    method = Method.GET,
    uri = hubServiceUri.withPath(s"${hubServiceUri.path}/node/${nodeKey.underlying}")
  )

  def getNodeRequest(hubServiceUri: Uri,nodeId:NodeId): Request[IO] = Request(
    method = Method.GET,
    uri = hubServiceUri.withPath(s"${hubServiceUri.path}/node/${nodeId.underlying}")
  )

  def getNodesRequest(hubServiceUri: Uri): Request[IO] = Request(
    method = Method.GET,
    uri = hubServiceUri.withPath(s"${hubServiceUri.path}/nodes")
  )

  def putNodeRequest(
                      hubServiceUri: Uri,
                      username:String,
                      password:String,
                      node:Node
                    ): Request[IO] = Request(
    method = Method.PUT,
    uri = hubServiceUri.withPath(s"${hubServiceUri.path}/node"),
    headers = Headers.of(Authorization(BasicCredentials(username,password)))
  ).withEntity(node.asJsonText.underlying)

  def createNodeRequest(
                         hubServiceUri: Uri,
                         key:String,
                         username:String,
                         password:String,
                         name:String,
                         userDomain:String,
                         adminEmail:String,
                         momQueue:Option[String] = None,
                         sendQueries:Option[Boolean] = None
                       ): Request[IO] = Request(
    method = Method.PUT,
    uri = hubServiceUri.
      withPath(s"${hubServiceUri.path}/createNode/$key").
      withQueryParam("nodeName",name).
      withQueryParam("userDomain",userDomain).
      withQueryParam("adminEmail", adminEmail).
      withOptionQueryParam("momQueueName",momQueue).
      withOptionQueryParam("sendQueries",sendQueries),
    headers = Headers.of(Authorization(BasicCredentials(username,password)))
  )

  def modifyNodeRequest(
                         hubServiceUri: Uri,
                         key:String,
                         username:String,
                         password:String,
                         name:Option[String] = None,
                         userDomain:Option[String] = None,
                         momQueue:Option[String] = None,
                         sendQueries:Option[Boolean] = None,
                         adminEmail:Option[Boolean] = None
                       ): Request[IO] = Request(
    method = Method.PUT,
    uri = hubServiceUri.
      withPath(s"${hubServiceUri.path}/modifyNode/$key").
      withOptionQueryParam("nodeName",name).
      withOptionQueryParam("userDomain",userDomain).
      withOptionQueryParam("momQueueName",momQueue).
      withOptionQueryParam("sendQueries",sendQueries).
      withOptionQueryParam("adminEmail",adminEmail),
    headers = Headers.of(Authorization(BasicCredentials(username,password)))
  )


  def modifyNetworkRequest(
                         hubServiceUri: Uri,
                         username:String,
                         password:String,
                         name:Option[String] = None,
                         momQueue:Option[String] = None
                       ): Request[IO] = Request(
    method = Method.PUT,
    uri = hubServiceUri.
      withPath(s"${hubServiceUri.path}/modifyNetwork").
      withOptionQueryParam("networkName",name).
      withOptionQueryParam("momQueueName",momQueue),
    headers = Headers.of(Authorization(BasicCredentials(username,password)))
  )


}
