package net.shrine.hub

import net.shrine.hub.data.store.HubDb
import net.shrine.protocol.version.{MomQueueName, NodeKey, NodeName, UserDomainName}
import net.shrine.protocol.version.v1.{Network, Node}
import net.shrine.messagequeueservice.MessageQueueService
import org.junit.{After, Before, Test}
import org.scalatestplus.junit.AssertionsForJUnit

class HubLifecycleTest extends AssertionsForJUnit {

  @Test
  def testOneNode():Unit = {

    val expectedNode = Node(new NodeName("Test Node"),new NodeKey("testNode"),new UserDomainName("testnode.domain"), adminEmail = "email@foo")
    HubDb.db.upsertNodeBlocking(expectedNode)

    {
      HubLifecycle.queuesFromDatabase().unsafeRunSync()
    }

    val queue = MessageQueueService.service.getQueueIO(expectedNode.momQueueName.underlying).unsafeRunSync()

    assertResult(queue.name)(expectedNode.momQueueName.underlying)
  }

  @Test
  def testTwoNodes():Unit = {
    val expectedNode1 = Node(new NodeName("Test Node"),new NodeKey("testNode"),new UserDomainName("testnode.domain"), adminEmail = "email@foo")
    HubDb.db.upsertNodeBlocking(expectedNode1)

    val expectedNode2 = Node(new NodeName("Test Node"),new NodeKey("testNode"),new UserDomainName("testnode.domain"), adminEmail = "email@foo")
    HubDb.db.upsertNodeBlocking(expectedNode2)

    {
      HubLifecycle.queuesFromDatabase().unsafeRunSync()
    }

    val queue1 = MessageQueueService.service.getQueueIO(expectedNode1.momQueueName.underlying).unsafeRunSync()
    assertResult(queue1.name)(expectedNode1.momQueueName.underlying)

    val queue2 = MessageQueueService.service.getQueueIO(expectedNode1.momQueueName.underlying).unsafeRunSync()
    assertResult(queue2.name)(expectedNode1.momQueueName.underlying)
  }

  @Before
  def beforeEach(): Unit = {
    HubDb.db.createTables()

    val expectedNetwork = Network(
      networkName = "testNetwork",
      hubQueueName = new MomQueueName("testHub"),
      adminEmail = "yourname@example.com"
    )
    HubDb.db.upsertNetworkBlocking(expectedNetwork)
  }

  @After
  def afterEach(): Unit = {
    HubDb.db.dropTables()
  }

}
