package net.shrine.messagequeueservice

import net.shrine.protocol.QueryResult
import net.shrine.protocol.version.v1.Node
import net.shrine.protocol.version.{Envelope, JsonText}
import org.junit.{Ignore, Test}

import scala.util.Try

final class MessageAsJsonTest {

  val testTimeoutMilliseconds = 10000L

  @Test
  def testMessageJsonRoundTrip() {
    val expectedMessageAsJson: MessageAsJson = MessageAsJson(new DeliveryAttemptId(-43),testTimeoutMilliseconds,3,"test message")
    val theMessageJson = expectedMessageAsJson.asJsonText

    val gotMessageAsJson = MessageAsJson.tryRead(theMessageJson).get
    assert(expectedMessageAsJson == gotMessageAsJson)
  }

  @Test
  def testWithEnvelope() {
    val expectedEnvelope = Envelope("testString",0L,"test envelope contents")
    val expectedEnvelopeJson = expectedEnvelope.asJsonText.underlying

    val expectedMessageAsJson: MessageAsJson = MessageAsJson(new DeliveryAttemptId(-43),testTimeoutMilliseconds,3,expectedEnvelopeJson)
    val theMessageJson = expectedMessageAsJson.asJsonText

    val gotMessageAsJson = MessageAsJson.tryRead(theMessageJson).get
    assert(expectedMessageAsJson == gotMessageAsJson)

    val gotEnvelopeAsJson = new JsonText(gotMessageAsJson.contents)
    val gotEnvelope = Envelope.tryRead(gotEnvelopeAsJson).get

    assert(expectedEnvelope == gotEnvelope)
  }

  @Test
  def testWithEnvelopeWithStuff() {
    val expectedNode = Node.create(
      name = "testNode",
      key = "testNode",
      userDomainName = "testNodeName",
      adminEmail = "email@foo"
    )

    val expectedEnvelope = Envelope(Node.envelopeType,0L,expectedNode.asJsonText.underlying)
    val expectedEnvelopeJson = expectedEnvelope.asJsonText.underlying

    val expectedMessageAsJson: MessageAsJson = MessageAsJson(new DeliveryAttemptId(-43),testTimeoutMilliseconds,3,expectedEnvelopeJson)
    val theMessageJson = expectedMessageAsJson.asJsonText

    val gotMessageAsJson = MessageAsJson.tryRead(theMessageJson).get
    assert(expectedMessageAsJson == gotMessageAsJson)

    val gotEnvelopeAsJson = new JsonText(gotMessageAsJson.contents)
    val gotEnvelope = Envelope.tryRead(gotEnvelopeAsJson).get

    assert(expectedEnvelope == gotEnvelope)

    val gotNode = Node.tryRead(new JsonText(gotEnvelope.contents)).get

    assert(expectedNode == gotNode)
  }
}

