package net.shrine.qep

import net.shrine.util.ShouldMatchersForJUnit
import org.junit.Test
import net.shrine.authentication.Authenticator
import net.shrine.protocol.{AuthenticationInfo, Credential, DefaultBreakdownResultOutputTypes, ReadResultOutputTypesRequest, ReadResultOutputTypesResponse, ResultOutputType, XmlNodeName}
import net.shrine.authentication.AuthenticationResult
import net.shrine.crypto.NewTestKeyStore
import net.shrine.i2b2.protocol.pm.User
import scala.concurrent.duration.DurationInt

/**
 * @author clint
 * @since Oct 23, 2014
 */
final class I2B2QepServiceTest extends ShouldMatchersForJUnit {

  private val knownUsername = "some-user"
  private val unknownUsername = "some-unknown-user"

  @Test
  def testReadResultOutputTypes(): Unit = {
    val authenticator: Authenticator = new Authenticator {
      override def authenticate(authn: AuthenticationInfo, onAuth: Option[User => Unit] = None): AuthenticationResult = {
        if (authn.username == knownUsername) {
          AuthenticationResult.Authenticated(authn.domain, authn.username)
        } else {
          AuthenticationResult.NotAuthenticated(authn.domain, authn.username, "blarg")
        }
      }
    }
    
    val breakdownResultOutputTypes = DefaultBreakdownResultOutputTypes.toSet

    val service = I2b2QepService(
      commonName = "example.com",
      authenticator = authenticator,
      authorizationService = null,
      breakdownTypes = breakdownResultOutputTypes,
      collectQepAudit = false,
      xmlNodeName = XmlNodeName("testNode"),
      certCollection = NewTestKeyStore.certCollection,
      sendToHubAttempts = 3
    )

    {
      val req = ReadResultOutputTypesRequest("project-id", 1.minute, AuthenticationInfo("d", knownUsername, Credential("foo", isToken = false)))

      val resp = service.readResultOutputTypes(req)

      resp.asInstanceOf[ReadResultOutputTypesResponse].outputTypes should equal(ResultOutputType.nonErrorTypes ++ breakdownResultOutputTypes)
    }

    {
      val req = ReadResultOutputTypesRequest("project-id", 1.minute, AuthenticationInfo("d", unknownUsername, Credential("foo", isToken = false)))

      intercept[Exception] {
        service.readResultOutputTypes(req)
      }
    }
  }
}