package net.shrine.api.dashboard

import net.shrine.http4s.catsio.ExecutionContexts
import net.shrine.problem.{Problems, XmlProblemDigest}

object DashboardStatusHelper {

  case class ProblemResponse(size: Int, offset: Int, n: Int, problems: Seq[XmlProblemDigest])

  def getProblems(offsetPreMod: Option[Int], nPreMax: Option[Int], epoch: Option[Long]): ProblemResponse = {

    implicit val executionContext = ExecutionContexts.databaseExecutionContext

    def floorMod(x: Int, y: Int) = {
      x - (x % y)
    }

    val db = Problems.DatabaseConnector

    val n = Math.max(0, nPreMax.getOrElse(20))
    val moddedOffset = floorMod(Math.max(0, offsetPreMod.getOrElse(0)), n)

    val query = for {
      result <- db.IO.sizeAndProblemDigest(n, moddedOffset)
    } yield (result._2, floorMod(Math.max(0, moddedOffset), n), n, result._1)

    val query2 = for {
      dateOffset <- db.IO.findIndexOfDate(epoch.getOrElse(0))
      moddedOffset = floorMod(dateOffset, n)
      result <- db.IO.sizeAndProblemDigest(n, moddedOffset)
    } yield (result._2, moddedOffset, n, result._1)

    val queryReal = if (epoch.isEmpty) query else query2
    val tupled = db.runBlocking(queryReal)
    val response: ProblemResponse = ProblemResponse(tupled._1, tupled._2, tupled._3, tupled._4)
    response
  }
}