package net.shrine.api.util

import com.typesafe.config.Config
import javax.sql.DataSource
import net.shrine.config.ConfigExtensions
import net.shrine.crypto.BouncyKeyStoreCollection
import net.shrine.dao.squeryl.{DataSourceSquerylInitializer, SquerylDbAdapterSelecter, SquerylInitializer}
import net.shrine.http4s.client.legacy.{EndpointConfig, Poster}
import net.shrine.protocol.{ResultOutputType, ResultOutputTypes, XmlNodeName}
import net.shrine.qep.QueryEntryPointComponents
import net.shrine.slick.TestableDataSourceCreator
import net.shrine.source.ConfigSource
import org.squeryl.internals.DatabaseAdapter


object ShrineConfiguration {

  val config: Config = ConfigSource.config
  val shrineConfig: Config = config.getConfig("shrine")

  val certCollection: BouncyKeyStoreCollection = BouncyKeyStoreCollection.fromConfig

  protected val squerylAdapter: DatabaseAdapter = SquerylDbAdapterSelecter.determineAdapter(shrineConfig.getString("shrineDatabaseType"))
  protected val dataSource: DataSource = TestableDataSourceCreator.dataSource(shrineConfig.getConfig("squerylDataSource.database"))

  protected val squerylInitializer: SquerylInitializer = new DataSourceSquerylInitializer(dataSource, squerylAdapter)

  private val pmEndpoint: EndpointConfig = shrineConfig.getConfigured("pmEndpoint", EndpointConfig(_))

  val breakdownTypes: Set[ResultOutputType] = shrineConfig.getOptionConfigured("breakdownResultOutputTypes", ResultOutputTypes.fromConfig).getOrElse(Set.empty)

  val pmPoster: Poster = Poster(certCollection,pmEndpoint)
  protected val xmlNodeName: XmlNodeName = XmlNodeName(shrineConfig.getString("nodeKey"))

  //todo delete a lot of this when possible
  val queryEntryPointComponents:Option[QueryEntryPointComponents] = shrineConfig.getOptionConfiguredIf(
    "queryEntryPoint",
    QueryEntryPointComponents(
      _,
    certCollection = certCollection,
    breakdownTypes = breakdownTypes,
    pmPoster = pmPoster, //todo could really have its own
    nodeId = xmlNodeName
  ))
}

